{ **************************************************************************** }
{ *                                                                          * }
{ * GALAXY II (Epoch) Simulator                                              * }
{ * by Luca "MADrigal" Antignano, September 2007 (revision: November 2022)   * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI, {SK} IniFiles;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;   {1, 2}
    im_background_screen_on:  TImage;                                           {3}

    btn_power_on:             TImage;       btn_power_top:            TImage;   {4, 5}
    btn_select_down:          TImage;       btn_select_top:           TImage;   {6, 7}
    btn_start_down:           TImage;       btn_start_top:            TImage;   {8, 9}

    btn_joystick_placeholder: TImage;       btn_joystick_top:         TImage;   {10, 11}
    btn_joystick_left:        TImage;                                           {12}
    btn_joystick_right:       TImage;                                           {13}

    btn_fire_down:            TImage;       btn_fire_top:             TImage;   {14, 15}

    btn_ctrlr_overlay_left:   TImage;                                           {16}
    btn_ctrlr_overlay_right:  TImage;                                           {17}

{ * VFD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (lives left, game mode, score, etc.)
    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; {18-21}

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage; {22-}
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {  -31}

    im_panel_letter_g: TImage; im_panel_letter_o: TImage; im_panel_letter_d: TImage; {32-34}

    // enemy ships: blank sprites
    im_enemy_01_09: TImage;       im_enemy_02_09: TImage;       im_enemy_03_09: TImage;       im_enemy_04_09: TImage;       im_enemy_05_09: TImage; {35-}
    im_enemy_01_08: TImage;       im_enemy_02_08: TImage;       im_enemy_03_08: TImage;       im_enemy_04_08: TImage;       im_enemy_05_08: TImage;
    im_enemy_01_07: TImage;       im_enemy_02_07: TImage;       im_enemy_03_07: TImage;       im_enemy_04_07: TImage;       im_enemy_05_07: TImage;
    im_enemy_01_06: TImage;       im_enemy_02_06: TImage;       im_enemy_03_06: TImage;       im_enemy_04_06: TImage;       im_enemy_05_06: TImage;
    im_enemy_01_05: TImage;       im_enemy_02_05: TImage;       im_enemy_03_05: TImage;       im_enemy_04_05: TImage;       im_enemy_05_05: TImage;
    im_enemy_01_04: TImage;       im_enemy_02_04: TImage;       im_enemy_03_04: TImage;       im_enemy_04_04: TImage;       im_enemy_05_04: TImage;
    im_enemy_01_03: TImage;       im_enemy_02_03: TImage;       im_enemy_03_03: TImage;       im_enemy_04_03: TImage;       im_enemy_05_03: TImage;
    im_enemy_01_02: TImage;       im_enemy_02_02: TImage;       im_enemy_03_02: TImage;       im_enemy_04_02: TImage;       im_enemy_05_02: TImage;
    im_enemy_01_01: TImage;       im_enemy_02_01: TImage;       im_enemy_03_01: TImage;       im_enemy_04_01: TImage;       im_enemy_05_01: TImage; {  -79}

    // enemy ships: whole sprites
    im_enemy_01_06_whole: TImage; im_enemy_02_06_whole: TImage; im_enemy_03_06_whole: TImage; im_enemy_04_06_whole: TImage; im_enemy_05_06_whole: TImage; {80-}
                                  im_enemy_02_05_whole: TImage; im_enemy_03_05_whole: TImage; im_enemy_04_05_whole: TImage;
    im_enemy_01_01_whole: TImage; im_enemy_02_01_whole: TImage; im_enemy_03_01_whole: TImage; im_enemy_04_01_whole: TImage; im_enemy_05_01_whole: TImage; {  -92}

    // enemy ships: body sprites
    im_enemy_01_06_body: TImage;  im_enemy_02_06_body: TImage;  im_enemy_03_06_body: TImage;  im_enemy_04_06_body: TImage;  im_enemy_05_06_body: TImage;  {93-}
                                  im_enemy_02_05_body: TImage;  im_enemy_03_05_body: TImage;  im_enemy_04_05_body: TImage;
    im_enemy_01_01_body: TImage;  im_enemy_02_01_body: TImage;  im_enemy_03_01_body: TImage;  im_enemy_04_01_body: TImage;  im_enemy_05_01_body: TImage;  {  -105}

    // enemy ships: wings sprites
    im_enemy_01_06_wings: TImage; im_enemy_02_06_wings: TImage; im_enemy_03_06_wings: TImage; im_enemy_04_06_wings: TImage; im_enemy_05_06_wings: TImage; {106-}
                                  im_enemy_02_05_wings: TImage; im_enemy_03_05_wings: TImage; im_enemy_04_05_wings: TImage;
    im_enemy_01_01_wings: TImage; im_enemy_02_01_wings: TImage; im_enemy_03_01_wings: TImage; im_enemy_04_01_wings: TImage; im_enemy_05_01_wings: TImage; {   -118}

    // bullets
    im_fire_01_09: TImage;        im_fire_02_09: TImage;        im_fire_03_09: TImage;        im_fire_04_09: TImage;        im_fire_05_09: TImage; {119-}
    im_fire_01_08: TImage;        im_fire_02_08: TImage;        im_fire_03_08: TImage;        im_fire_04_08: TImage;        im_fire_05_08: TImage;
    im_fire_01_07: TImage;        im_fire_02_07: TImage;        im_fire_03_07: TImage;        im_fire_04_07: TImage;        im_fire_05_07: TImage;
    im_fire_01_06: TImage;        im_fire_02_06: TImage;        im_fire_03_06: TImage;        im_fire_04_06: TImage;        im_fire_05_06: TImage;
    im_fire_01_05: TImage;        im_fire_02_05: TImage;        im_fire_03_05: TImage;        im_fire_04_05: TImage;        im_fire_05_05: TImage;
    im_fire_01_04: TImage;        im_fire_02_04: TImage;        im_fire_03_04: TImage;        im_fire_04_04: TImage;        im_fire_05_04: TImage;
    im_fire_01_03: TImage;        im_fire_02_03: TImage;        im_fire_03_03: TImage;        im_fire_04_03: TImage;        im_fire_05_03: TImage;
    im_fire_01_02: TImage;        im_fire_02_02: TImage;        im_fire_03_02: TImage;        im_fire_04_02: TImage;        im_fire_05_02: TImage; {   -158}

    // player's ship: blank sprites
    im_ship_01_01: TImage;        im_ship_02_01: TImage;        im_ship_03_01: TImage;        im_ship_04_01: TImage;        im_ship_05_01: TImage; {159=163}

    // player's ship: whole sprites
    im_ship_01_01_whole: TImage;  im_ship_02_01_whole: TImage;  im_ship_03_01_whole: TImage;  im_ship_04_01_whole: TImage;  im_ship_05_01_whole: TImage; {165-168}
    im_ship_01_01_dock: TImage;   im_ship_02_01_dock: TImage;   im_ship_03_01_dock: TImage;   im_ship_04_01_dock: TImage;   im_ship_05_01_dock: TImage;  {169-173}

{ * DEBUGGING ITEMS                                                          * }
    la_debug_aEnemy: TLabel;
    la_debug_aFire:  TLabel;

{ * TIMERS                                                                   * }

    // timers: non-ingame
    Timer_Demo_1:                         TTimer;
    Timer_Demo_2:                         TTimer;
    Timer_Game_Start:                     TTimer;
    Timer_Game_Over:                      TTimer;

    // timers: used in all phases
    Timer_Keypress:                       TTimer;

    // timers: used in phases 1, 2 and 3
    Timer_Phase123_Player_Fire:           TTimer;
    Timer_Phase123_Miss:                  TTimer;
    Timer_Phase123_Enemy_Fire:            TTimer;

    // timers: used in phase 1 only
    Timer_Phase1_Enemy_Row8:              TTimer;
    Timer_Phase1_Enemy_Row7:              TTimer;
    Timer_Phase1_Enemy_Attacking:         TTimer;
    Timer_Phase1_Enemy_Rows78_Hit:        TTimer;
    Timer_Phase1_Enemy_Attacking_Hit:     TTimer;
    Timer_Phase1_Reprise:                 TTimer;
    Timer_Phase1_Complete:                TTimer;

    // timers: used in phase 2 only
    Timer_Phase2_Enemy_Attacking:         TTimer;
    Timer_Phase2_Reprise:                 TTimer;
    Timer_Phase2_Enemy_Attacking_Hit:     TTimer;
    Timer_Phase2_Complete:                TTimer;

    // timers: used in phase 3 only
    Timer_Phase3_Mothership:              TTimer;
    Timer_Phase3_Reprise:                 TTimer;
    Timer_Phase3_Mothership_Hit:          TTimer;
    Timer_Phase3_Complete:                TTimer;

    // timers: used in phase 4 only
    Timer_Phase4_Shuttle_Thrust:          TTimer;
    Timer_Phase4_Shuttle_Thrust_Display:  TTimer;
    Timer_Phase4_Shuttle_Down:            TTimer;
    Timer_Phase4_Dock:                    TTimer;
    Timer_Phase4_Miss_Dock:               TTimer;
    Timer_Phase4_Bonus:                   TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                  (Sender: TObject);
    procedure FormActivate                (Sender: TObject);
    procedure FormClose                   (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Display_All_Sprites     (b: boolean);

    procedure Vid_Display_Ship            (b: boolean);
    procedure Vid_Display_Shuttle         (b: boolean);
    procedure Vid_Display_Dock            (b: boolean);

    procedure Vid_Display_Panel_Score     (var p: integer; b: boolean);
    procedure Vid_Display_Panel_Skill     (var p: integer; b: boolean);
    procedure Vid_Display_Panel_Lives     (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                    (pfs: PfSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove      (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_power_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_select_topMouseDown     (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_select_topMouseUp       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_start_topMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_start_topMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_fire_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_fire_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject;Button:TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button:TMouseButton;Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                 (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                   (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure inp_power_down;
    procedure inp_select_down;            procedure inp_select_up;
    procedure inp_start_down;             procedure inp_start_up;

    procedure inp_left_down;              procedure inp_left_up;
    procedure inp_right_down;             procedure inp_right_up;
    procedure inp_fire_down;              procedure inp_fire_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Reset_Numeric_Arrays;
    procedure Gam_Set_Mode;

    procedure Gam_Phase123_Set_Sprites;
    procedure Gam_Phase123_Check_Collision;
    procedure Gam_Phase123_Miss;
    procedure Gam_Phase4_Set_Sprites;
    procedure Gam_Phase4_Check_Dock;

{ * PROCEDURES: DEBUGGER                                                     * }
    procedure Debug_aEnemy;
    procedure Debug_aFire;

{ * PROCEDURES: TIMERS: NON-INGAME                                           * }
    procedure Timer_Demo_1Timer                        (Sender: TObject);
    procedure Timer_Demo_2Timer                        (Sender: TObject);
    procedure Timer_Game_StartTimer                    (Sender: TObject);
    procedure Timer_Game_OverTimer                     (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN ALL PHASES                                   * }
    procedure Timer_KeypressTimer                      (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASES 1, 2 AND 3                            * }
    procedure Timer_Phase123_Player_FireTimer          (Sender: TObject);
    procedure Timer_Phase123_Enemy_FireTimer           (Sender: TObject);
    procedure Timer_Phase123_MissTimer                 (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 1 ONLY                                 * }
    procedure Timer_Phase1_Enemy_Row8Timer             (Sender: TObject);
    procedure Timer_Phase1_Enemy_Row7Timer             (Sender: TObject);
    procedure Timer_Phase1_Enemy_Rows78_HitTimer       (Sender: TObject);
    procedure Timer_Phase1_Enemy_AttackingTimer        (Sender: TObject);
    procedure Timer_Phase1_Enemy_Attacking_HitTimer    (Sender: TObject);
    procedure Timer_Phase1_RepriseTimer                (Sender: TObject);
    procedure Timer_Phase1_CompleteTimer               (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 2 ONLY                                 * }
    procedure Timer_Phase2_Enemy_AttackingTimer        (Sender: TObject);
    procedure Timer_Phase2_Enemy_Attacking_HitTimer    (Sender: TObject);
    procedure Timer_Phase2_RepriseTimer                (Sender: TObject);
    procedure Timer_Phase2_CompleteTimer               (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 3 ONLY                                 * }
    procedure Timer_Phase3_MothershipTimer             (Sender: TObject);
    procedure Timer_Phase3_RepriseTimer                (Sender: TObject);
    procedure Timer_Phase3_Mothership_HitTimer         (Sender: TObject);
    procedure Timer_Phase3_CompleteTimer               (Sender: TObject);

{ * PROCEDURES: TIMERS: USED IN PHASE 4 ONLY                                 * }
    procedure Timer_Phase4_Shuttle_ThrustTimer         (Sender: TObject);
    procedure Timer_Phase4_Shuttle_Thrust_DisplayTimer (Sender :TObject);
    procedure Timer_Phase4_Shuttle_DownTimer           (Sender: TObject);
    procedure Timer_Phase4_DockTimer                   (Sender: TObject);
    procedure Timer_Phase4_Miss_DockTimer              (Sender: TObject);
    procedure Timer_Phase4_BonusTimer                  (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Galaxy II (Epoch)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/11/21';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
{SK}INIFILE       = 'Config.ini';                                               {SK: now exclusively related to the executable configuration file pointing to the various skins}

  APP_NAME      = HH2_CONTENT_NAME + ' Simulator';                              // application's name (appears on taskbar and Splash screen)
{}APP_VERSION   = 'Version ' + HH2_CONTENT_VERSION + ' * Multi-skin System + bug fix *';  // current version (appears on Splash screen)
{}APP_DATE      = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 21st November, 2022'; // release date (appears on Splash screen)

  URL_EMAIL     = 'mailto:' + HH2_CONTENT_EMAIL;                                // link to the email address
  URL_PAYPAL    = 'https://www.paypal.me/LucaAntignano';                        // link to the Paypal donation page
  URL_WEBSITE   = 'http://' + HH2_CONTENT_WEBSITE + '/';                        // link to the website

  REG_KEY       = 'Software\MADrigal\GalaxyII50';                               // registry key

  HINT_CLICK    = 'click or press';
  HINT_PWR_OFF  = 'to turn on the game';
  HINT_PWR_ON   = 'to turn off the game';
  HINT_SELECT   = 'to select game';
  HINT_START    = 'to start game';
  HINT_JOYSTICK = 'activate the controller overlays or use';

{}MENU_ITEMS = 13;
  NUM_KEYS = 6;

  MAX_ENEMIES            = 10;                                                  // number of enemy ships to kill in phase 1 and 2

  PHASE123_KEYPRESS      = 300;
  PHASE4_KEYPRESS        = 800;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save settings)

{SK}{ * inifiles management * }
{SK}iNumSkins,                                                                  // number of skins present as read from the main 'config.ini' file
{SK}iSelectedSkin          : integer;                                           // index of selected skin (as result of combobox in Splash Screen
{SK}
{SK}aSkin                  : array[1..99] of record                             // how many skins present in the main 'config.ini' file
{SK}                                           Path,
{SK}                                           Name,
{SK}                                           SkinFile,
{SK}                                           SkinPreviewImg: string;
{SK}                                         end;

{ * sound management * }
  pfs_bonus,
  pfs_complete,
  pfs_fire,
  pfs_gameon,
  pfs_hit,
  pfs_miss,
  pfs_missdock,
  pfs_gameover,
  pfs_phase,
  pfs_selecbtn,
  pfs_start,
  pfs_startbtn,
  pfs_tick,
  pfs_thrust:              PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score display

  aIm_Enemy:               array[1..5, 1..9] of TImage;                         // empty sprites to host enemy pictures
  aIm_Enemy_Picture:       array[1..5, 1..6] of record                          // pictures of enemies
                                                  Whole: TImage;                  // body + wings
                                                  Body:  TImage;                  // body only
                                                  Wings: TImage;                  // wings only
                                                end;

  aIm_Fire:                array[1..5, 2..9] of TImage;                         // bullets (both player's and enemies')

  aIm_Ship:                array[1..5]       of TImage;                         // player's space ship
  aIm_Ship_Picture:        array[1..5]       of record                          // pictures of player's shop
                                                  Whole: TImage;                  // dock + shuttle
                                                  Dock:  TImage;                  // dock only
                                                end;

{ * arrays of mixed types *}
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                                   int:  integer;
                                                   word: word;
                                                 end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * arrays of numbers *}
  aEnemy:                  array[1..5, 1..9] of integer;                        // 0 (empty); 1 (enemy ships in rows 7, 8 or 9); 2 (1st attacking enemy ship); 4 (2nd attacking enemy ship)

  aEnemy6:                 array[1..2]       of record                          // attacking enemy ships
                                                  iPosX,
                                                  iPosY,
                                                  iDirection,                   // depending on the value of iPhase, suggest which is the direction of the attacking enemy ship
                                                  iValue:     integer;          // score value
                                                end;

  aEnemy9:                 array[1..5]       of integer;                        // temp array used in TimerPhase3Reprise

  aFire:                   array[1..5, 2..9] of integer;                        // 0 (empty); 1 (bullet going down); 2 (bullet going up)

  aGameSpeed:              array[1..4]       of integer;                        // overall speed for game timers (in game phases 1-3)

{ * integers and booleans * }
  iMode,                                                                        // 0 = game off; 1 = demo (1/2); 2 = demo (2/2) ; 3 = game
  iSkill,                                                                       // game difficulty (1-4)
  iPhase,                                                                       // phase (1-4)
  iLives,                                                                       // lives left (0-5)
  iEnemiesLeft,                                                                 // enemies to kill to complete phase
  iScore,                                                                       // score (0-9999 then "good")
  iBonus,                                                                       // bonus for phase 4 (0-999)
  iShipX,                                                                       // x coordinate of player's ship/shuttle
  iShipY,                                                                       // y coordinate of player's shuttle in phase 4
  iDockX,                                                                       // x coordinate of player's dock in phase 4
  iBulletX,                                                                     // x coordinate of player's ship bullet in phases 1-3
  iEnemyHitX,
  iEnemyHitY
                         : integer;

  // Game variables: loops
  iLoop,
  iLoopHit,
  iLoopDisplayThrust,
  iLoopDock
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bControllers,
  bJoyPressed,
  bFirePressed,
  bBulletOnScreen,
  bPhase4SndPlay
                         : boolean;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i, j: integer;
{SK}bError: boolean;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

{SK}// skin configuration files management
{SK}bError := false;

{SK}if not FileExists(ExtractFilePath(ParamStr(0)) + INIFILE) then
{SK}  bError := true
{SK}else
{SK}  begin
{SK}    IF_File := TIniFile.Create(ExtractFilePath(ParamStr(0)) + INIFILE);
{SK}
{SK}    iNumSkins := IF_file.ReadInteger('Common', 'Skins', 2);
{SK}
{SK}    for i := 1 to iNumSkins do
{SK}      begin
{SK}        aSkin[i].Path := IncludeTrailingPathDelimiter(IF_file.ReadString('Skin' + IntToStr(i), 'Path', 'Graphics\Skin' + IntToStr(i) + '\'));
{SK}        aSkin[i].Name := IF_file.ReadString('Skin' + IntToStr(i), 'Name', 'Nameless Skin');
{SK}        aSkin[i].SkinFile := IF_file.ReadString('Skin' + IntToStr(i), 'SkinFile', 'Skin.ini');
{SK}        aSkin[i].SkinPreviewImg := IF_file.ReadString('Skin' + IntToStr(i), 'SkinPreviewImg', 'SkinPreview.png');
{SK}      end;
{SK}    IF_File.Destroy;
{SK}  end;
{SK}
{SK}if bError then
{SK}  begin
{SK}    MessageDlg('Configuration file (' + INIFILE + ') not found. Please reinstall the game.', mtError, [mbOK], 0);
{SK}    Application.Terminate;
{SK}  end;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // enemies
  aIm_Enemy[1, 9] := im_enemy_01_09;                aIm_Enemy[2, 9] := im_enemy_02_09;                aIm_Enemy[3, 9] := im_enemy_03_09;                aIm_Enemy[4, 9] := im_enemy_04_09;                aIm_Enemy[5, 9] := im_enemy_05_09;
  aIm_Enemy[1, 8] := im_enemy_01_08;                aIm_Enemy[2, 8] := im_enemy_02_08;                aIm_Enemy[3, 8] := im_enemy_03_08;                aIm_Enemy[4, 8] := im_enemy_04_08;                aIm_Enemy[5, 8] := im_enemy_05_08;
  aIm_Enemy[1, 7] := im_enemy_01_07;                aIm_Enemy[2, 7] := im_enemy_02_07;                aIm_Enemy[3, 7] := im_enemy_03_07;                aIm_Enemy[4, 7] := im_enemy_04_07;                aIm_Enemy[5, 7] := im_enemy_05_07;
  aIm_Enemy[1, 6] := im_enemy_01_06;                aIm_Enemy[2, 6] := im_enemy_02_06;                aIm_Enemy[3, 6] := im_enemy_03_06;                aIm_Enemy[4, 6] := im_enemy_04_06;                aIm_Enemy[5, 6] := im_enemy_05_06;
  aIm_Enemy[1, 5] := im_enemy_01_05;                aIm_Enemy[2, 5] := im_enemy_02_05;                aIm_Enemy[3, 5] := im_enemy_03_05;                aIm_Enemy[4, 5] := im_enemy_04_05;                aIm_Enemy[5, 5] := im_enemy_05_05;
  aIm_Enemy[1, 4] := im_enemy_01_04;                aIm_Enemy[2, 4] := im_enemy_02_04;                aIm_Enemy[3, 4] := im_enemy_03_04;                aIm_Enemy[4, 4] := im_enemy_04_04;                aIm_Enemy[5, 4] := im_enemy_05_04;
  aIm_Enemy[1, 3] := im_enemy_01_03;                aIm_Enemy[2, 3] := im_enemy_02_03;                aIm_Enemy[3, 3] := im_enemy_03_03;                aIm_Enemy[4, 3] := im_enemy_04_03;                aIm_Enemy[5, 3] := im_enemy_05_03;
  aIm_Enemy[1, 2] := im_enemy_01_02;                aIm_Enemy[2, 2] := im_enemy_02_02;                aIm_Enemy[3, 2] := im_enemy_03_02;                aIm_Enemy[4, 2] := im_enemy_04_02;                aIm_Enemy[5, 2] := im_enemy_05_02;
  aIm_Enemy[1, 1] := im_enemy_01_01;                aIm_Enemy[2, 1] := im_enemy_02_01;                aIm_Enemy[3, 1] := im_enemy_03_01;                aIm_Enemy[4, 1] := im_enemy_04_01;                aIm_Enemy[5, 1] := im_enemy_05_01;

  // bullets
  aIm_Fire[1, 9] := im_fire_01_09;                  aIm_Fire[2, 9] := im_fire_02_09;                  aIm_Fire[3, 9] := im_fire_03_09;                  aIm_Fire[4, 9] := im_fire_04_09;                  aIm_Fire[5, 9] := im_fire_05_09;
  aIm_Fire[1, 8] := im_fire_01_08;                  aIm_Fire[2, 8] := im_fire_02_08;                  aIm_Fire[3, 8] := im_fire_03_08;                  aIm_Fire[4, 8] := im_fire_04_08;                  aIm_Fire[5, 8] := im_fire_05_08;
  aIm_Fire[1, 7] := im_fire_01_07;                  aIm_Fire[2, 7] := im_fire_02_07;                  aIm_Fire[3, 7] := im_fire_03_07;                  aIm_Fire[4, 7] := im_fire_04_07;                  aIm_Fire[5, 7] := im_fire_05_07;
  aIm_Fire[1, 6] := im_fire_01_06;                  aIm_Fire[2, 6] := im_fire_02_06;                  aIm_Fire[3, 6] := im_fire_03_06;                  aIm_Fire[4, 6] := im_fire_04_06;                  aIm_Fire[5, 6] := im_fire_05_06;
  aIm_Fire[1, 5] := im_fire_01_05;                  aIm_Fire[2, 5] := im_fire_02_05;                  aIm_Fire[3, 5] := im_fire_03_05;                  aIm_Fire[4, 5] := im_fire_04_05;                  aIm_Fire[5, 5] := im_fire_05_05;
  aIm_Fire[1, 4] := im_fire_01_04;                  aIm_Fire[2, 4] := im_fire_02_04;                  aIm_Fire[3, 4] := im_fire_03_04;                  aIm_Fire[4, 4] := im_fire_04_04;                  aIm_Fire[5, 4] := im_fire_05_04;
  aIm_Fire[1, 3] := im_fire_01_03;                  aIm_Fire[2, 3] := im_fire_02_03;                  aIm_Fire[3, 3] := im_fire_03_03;                  aIm_Fire[4, 3] := im_fire_04_03;                  aIm_Fire[5, 3] := im_fire_05_03;
  aIm_Fire[1, 2] := im_fire_01_02;                  aIm_Fire[2, 2] := im_fire_02_02;                  aIm_Fire[3, 2] := im_fire_03_02;                  aIm_Fire[4, 2] := im_fire_04_02;                  aIm_Fire[5, 2] := im_fire_05_02;

  // player's ship
  aIm_Ship[1] := im_ship_01_01;                     aIm_Ship[2] := im_ship_02_01;                     aIm_Ship[3] := im_ship_03_01;                     aIm_Ship[4] := im_ship_04_01;                     aIm_Ship[5] := im_ship_05_01;

  // enemies sprites (whole sprite, bodies only, wings only)
  aIm_Enemy_Picture[1, 6].Whole := im_enemy_01_06_whole; aIm_Enemy_Picture[2, 6].Whole := im_enemy_02_06_whole; aIm_Enemy_Picture[3, 6].Whole := im_enemy_03_06_whole; aIm_Enemy_Picture[4, 6].Whole := im_enemy_04_06_whole; aIm_Enemy_Picture[5, 6].Whole := im_enemy_05_06_whole;
  aIm_Enemy_Picture[1, 5].Whole := im_enemy_01_06_whole; aIm_Enemy_Picture[2, 5].Whole := im_enemy_02_05_whole; aIm_Enemy_Picture[3, 5].Whole := im_enemy_03_05_whole; aIm_Enemy_Picture[4, 5].Whole := im_enemy_04_05_whole; aIm_Enemy_Picture[5, 5].Whole := im_enemy_05_06_whole;
  aIm_Enemy_Picture[1, 4].Whole := im_enemy_01_06_whole; aIm_Enemy_Picture[2, 4].Whole := im_enemy_02_06_whole; aIm_Enemy_Picture[3, 4].Whole := im_enemy_03_06_whole; aIm_Enemy_Picture[4, 4].Whole := im_enemy_04_06_whole; aIm_Enemy_Picture[5, 4].Whole := im_enemy_05_06_whole;
  aIm_Enemy_Picture[1, 3].Whole := im_enemy_01_06_whole; aIm_Enemy_Picture[2, 3].Whole := im_enemy_02_05_whole; aIm_Enemy_Picture[3, 3].Whole := im_enemy_03_05_whole; aIm_Enemy_Picture[4, 3].Whole := im_enemy_04_05_whole; aIm_Enemy_Picture[5, 3].Whole := im_enemy_05_06_whole;
  aIm_Enemy_Picture[1, 2].Whole := im_enemy_01_06_whole; aIm_Enemy_Picture[2, 2].Whole := im_enemy_02_06_whole; aIm_Enemy_Picture[3, 2].Whole := im_enemy_03_06_whole; aIm_Enemy_Picture[4, 2].Whole := im_enemy_04_06_whole; aIm_Enemy_Picture[5, 2].Whole := im_enemy_05_06_whole;
  aIm_Enemy_Picture[1, 1].Whole := im_enemy_01_01_whole; aIm_Enemy_Picture[2, 1].Whole := im_enemy_02_01_whole; aIm_Enemy_Picture[3, 1].Whole := im_enemy_01_01_whole; aIm_Enemy_Picture[4, 1].Whole := im_enemy_02_01_whole; aIm_Enemy_Picture[5, 1].Whole := im_enemy_01_01_whole;

  aIm_Enemy_Picture[1, 6].Body := im_enemy_01_06_body;   aIm_Enemy_Picture[2, 6].Body := im_enemy_02_06_body;   aIm_Enemy_Picture[3, 6].Body := im_enemy_03_06_body;   aIm_Enemy_Picture[4, 6].Body := im_enemy_04_06_body;   aIm_Enemy_Picture[5, 6].Body := im_enemy_05_06_body;
  aIm_Enemy_Picture[1, 5].Body := im_enemy_01_06_body;   aIm_Enemy_Picture[2, 5].Body := im_enemy_02_05_body;   aIm_Enemy_Picture[3, 5].Body := im_enemy_03_05_body;   aIm_Enemy_Picture[4, 5].Body := im_enemy_04_05_body;   aIm_Enemy_Picture[5, 5].Body := im_enemy_05_06_body;
  aIm_Enemy_Picture[1, 4].Body := im_enemy_01_06_body;   aIm_Enemy_Picture[2, 4].Body := im_enemy_02_06_body;   aIm_Enemy_Picture[3, 4].Body := im_enemy_03_06_body;   aIm_Enemy_Picture[4, 4].Body := im_enemy_04_06_body;   aIm_Enemy_Picture[5, 4].Body := im_enemy_05_06_body;
  aIm_Enemy_Picture[1, 3].Body := im_enemy_01_06_body;   aIm_Enemy_Picture[2, 3].Body := im_enemy_02_05_body;   aIm_Enemy_Picture[3, 3].Body := im_enemy_03_05_body;   aIm_Enemy_Picture[4, 3].Body := im_enemy_04_05_body;   aIm_Enemy_Picture[5, 3].Body := im_enemy_05_06_body;
  aIm_Enemy_Picture[1, 2].Body := im_enemy_01_06_body;   aIm_Enemy_Picture[2, 2].Body := im_enemy_02_06_body;   aIm_Enemy_Picture[3, 2].Body := im_enemy_03_06_body;   aIm_Enemy_Picture[4, 2].Body := im_enemy_04_06_body;   aIm_Enemy_Picture[5, 2].Body := im_enemy_05_06_body;
  aIm_Enemy_Picture[1, 1].Body := im_enemy_01_01_body;   aIm_Enemy_Picture[2, 1].Body := im_enemy_02_01_body;   aIm_Enemy_Picture[3, 1].Body := im_enemy_01_01_body;   aIm_Enemy_Picture[4, 1].Body := im_enemy_02_01_body;   aIm_Enemy_Picture[5, 1].Body := im_enemy_01_01_body;

  aIm_Enemy_Picture[1, 6].Wings := im_enemy_01_06_wings; aIm_Enemy_Picture[2, 6].Wings := im_enemy_02_06_wings; aIm_Enemy_Picture[3, 6].Wings := im_enemy_03_06_wings; aIm_Enemy_Picture[4, 6].Wings := im_enemy_04_06_wings; aIm_Enemy_Picture[5, 6].Wings := im_enemy_05_06_wings;
  aIm_Enemy_Picture[1, 5].Wings := im_enemy_01_06_wings; aIm_Enemy_Picture[2, 5].Wings := im_enemy_02_05_wings; aIm_Enemy_Picture[3, 5].Wings := im_enemy_03_05_wings; aIm_Enemy_Picture[4, 5].Wings := im_enemy_04_05_wings; aIm_Enemy_Picture[5, 5].Wings := im_enemy_05_06_wings;
  aIm_Enemy_Picture[1, 4].Wings := im_enemy_01_06_wings; aIm_Enemy_Picture[2, 4].Wings := im_enemy_02_06_wings; aIm_Enemy_Picture[3, 4].Wings := im_enemy_03_06_wings; aIm_Enemy_Picture[4, 4].Wings := im_enemy_04_06_wings; aIm_Enemy_Picture[5, 4].Wings := im_enemy_05_06_wings;
  aIm_Enemy_Picture[1, 3].Wings := im_enemy_01_06_wings; aIm_Enemy_Picture[2, 3].Wings := im_enemy_02_05_wings; aIm_Enemy_Picture[3, 3].Wings := im_enemy_03_05_wings; aIm_Enemy_Picture[4, 3].Wings := im_enemy_04_05_wings; aIm_Enemy_Picture[5, 3].Wings := im_enemy_05_06_wings;
  aIm_Enemy_Picture[1, 2].Wings := im_enemy_01_06_wings; aIm_Enemy_Picture[2, 2].Wings := im_enemy_02_06_wings; aIm_Enemy_Picture[3, 2].Wings := im_enemy_03_06_wings; aIm_Enemy_Picture[4, 2].Wings := im_enemy_04_06_wings; aIm_Enemy_Picture[5, 2].Wings := im_enemy_05_06_wings;
  aIm_Enemy_Picture[1, 1].Wings := im_enemy_01_01_wings; aIm_Enemy_Picture[2, 1].Wings := im_enemy_02_01_wings; aIm_Enemy_Picture[3, 1].Wings := im_enemy_01_01_wings; aIm_Enemy_Picture[4, 1].Wings := im_enemy_02_01_wings; aIm_Enemy_Picture[5, 1].Wings := im_enemy_01_01_wings;

  // player's ship (whole sprite, dock only)
  aIm_Ship_Picture[1].Whole := im_ship_01_01_whole;      aIm_Ship_Picture[2].Whole := im_ship_02_01_whole;      aIm_Ship_Picture[3].Whole := im_ship_03_01_whole;      aIm_Ship_Picture[4].Whole := im_ship_04_01_whole;      aIm_Ship_Picture[5].Whole := im_ship_05_01_whole;
  aIm_Ship_Picture[1].Dock := im_ship_01_01_dock;        aIm_Ship_Picture[2].Dock := im_ship_02_01_dock;        aIm_Ship_Picture[3].Dock := im_ship_03_01_dock;        aIm_Ship_Picture[4].Dock := im_ship_04_01_dock;        aIm_Ship_Picture[5].Dock := im_ship_05_01_dock;

  // arrays of integers (difficulty settings)
  aGameSpeed[1] := 400;
  aGameSpeed[2] := 240;
  aGameSpeed[3] := 400;
  aGameSpeed[4] := 240;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{SK: clause "Gfx_Game_Init;" moved into the 'Button Play Click' procedure in splash.pas
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Power On/Off" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Select"       (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Start"        (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 39);                             // "Left"         (default: "LEFT")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 40);                             // "Right"        (default: "RIGHT")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 42);                             // "Fire"         (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_power_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "Power off"
  btn_select_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_SELECT;  // "Select"
  btn_start_top.Hint           := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_START;   // "Start"

  btn_joystick_top.Hint        := HINT_JOYSTICK + ' ' + aKeyHints[aKeys[4].int] + ', ' // "Left"
                                                      + aKeyHints[aKeys[5].int];       // "Right"

  btn_fire_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];   // "Fire"

  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];   // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];   // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_9_n2;
{}aMenuItem[9]  := pm_10_fullscreen;
{}aMenuItem[10] := pm_11_splashscreen;
{}aMenuItem[11] := pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  bFirePressed := false;
  bJoyPressed  := false;
  bControllers := false;

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_bonus);    FSOUND_Sample_Free(pfs_complete);
  FSOUND_Sample_Free(pfs_fire);     FSOUND_Sample_Free(pfs_gameon);
  FSOUND_Sample_Free(pfs_hit);      FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_missdock); FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_phase);    FSOUND_Sample_Free(pfs_selecbtn);
  FSOUND_Sample_Free(pfs_start);    FSOUND_Sample_Free(pfs_startbtn);
  FSOUND_Sample_Free(pfs_thrust);   FSOUND_Sample_Free(pfs_tick);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;

  btn_joystick_top.Visible := (not bDisplayCtrlOverlays);
end;





procedure TForm1.Vid_Display_All_Sprites(b: boolean);
{ displays/hides all sprites, incl. score panel                                }
  var
    i, j: integer;
begin
  for i := 1 to 4 do
    begin
      aIm_panel_digit_pos[i].Visible := true;

      if b then
        aIm_panel_digit_pos[i].Picture := aIm_panel_number[8].Picture
      else
        aIm_panel_digit_pos[i].Picture := nil;
    end;

  for i := 1 to 5 do
    begin
      for j := 1 to 9 do aIm_Enemy[i, j].Visible := b;
      for j := 2 to 9 do aIm_Fire[i, j].Visible  := b;
                         aIm_Ship[i].Visible     := b;
    end;
end;





procedure TForm1.Vid_Display_Ship(b: boolean);
{ displays/hides the player's ship (in game phases 1 to 3                      }
begin
  aIm_Ship[iShipX].Visible := b;
end;





procedure TForm1.Vid_Display_Shuttle(b: boolean);
{ displays/hides the player's shuttle (in game phase 4)                        }
begin
  aIm_Enemy[iShipX, iShipY].Visible := b;
end;





procedure TForm1.Vid_Display_Dock(b: boolean);
{ displays/hides the dock (in game phase 4)                                    }
begin
  aIm_Ship[iDockX].Visible := b;
end;





procedure TForm1.Vid_Display_Panel_Score(var p: integer; b: boolean);
{ transforms score into sprites, and displays/hides                            }
  var
    i: integer;
begin
  if b then                                                                     // display or hide number?
    if p > 9999 then                                                              // display
      begin                                                                         // 9999+ ("good")
        aIm_panel_digit_pos[1].Picture := im_panel_letter_g.Picture;
        aIm_panel_digit_pos[2].Picture := im_panel_letter_o.Picture;
        aIm_panel_digit_pos[3].Picture := im_panel_letter_o.Picture;
        aIm_panel_digit_pos[4].Picture := im_panel_letter_d.Picture;
      end
    else                                                                            // 0-9999
      begin
        if p > 999 then aIm_panel_digit_pos[1].picture := aIm_panel_number[p div 1000 mod 10].picture else aIm_panel_digit_pos[1].picture := nil; // thousands
        if p > 99  then aIm_panel_digit_pos[2].picture := aIm_panel_number[p div  100 mod 10].picture else aIm_panel_digit_pos[2].picture := nil; // hundreds
        if p > 9   then aIm_panel_digit_pos[3].picture := aIm_panel_number[p div   10 mod 10].picture else aIm_panel_digit_pos[3].picture := nil; // tens
                        aIm_panel_digit_pos[4].picture := aIm_panel_number[p          mod 10].picture;                                            // units
      end
  else                                                                            // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;





procedure TForm1.Vid_Display_Panel_Skill(var p: integer; b: boolean);
{ transforms skill level into sprites, and displays/hides                      }
  var
    i: integer;
begin
  if b then                                                                     // display or hide number?
    begin                                                                         // display
      aIm_panel_digit_pos[1].picture := nil;
      aIm_panel_digit_pos[2].picture := nil;
      aIm_panel_digit_pos[3].picture := aIm_panel_number[iSkill].picture;
      aIm_panel_digit_pos[4].picture := nil;
    end
  else                                                                            // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;





procedure TForm1.Vid_Display_Panel_Lives(var p: integer; b: boolean);
{ transforms lives left into sprites, and displays/hides                       }
  var
    i: integer;
begin
  if b then                                                                     // display or hide number?
    begin                                                                         // display
      aIm_panel_digit_pos[1].picture := nil;
      aIm_panel_digit_pos[2].picture := nil;
      aIm_panel_digit_pos[3].picture := nil;
      aIm_panel_digit_pos[4].picture := aIm_panel_number[iLives].picture;
    end
  else                                                                            // hide
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(0, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_power_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ power button: down (click)                                                   }
begin
  if Button = mbLeft then
    inp_power_down;
end;





procedure TForm1.btn_select_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ select button: DOWN                                                          }
begin
  if Button = mbLeft then
    inp_select_down;
end;




procedure TForm1.btn_select_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ select button: UP                                                            }
begin
  if Button = mbLeft then
    inp_select_up;
end;





procedure TForm1.btn_start_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ start button: DOWN                                                           }
begin
  if Button = mbLeft then
    inp_start_down;
end;





procedure TForm1.btn_start_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ start button: UP                                                             }
begin
  if Button = mbLeft then
    inp_start_up;
end;





procedure TForm1.btn_fire_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ fire button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_fire_down;
end;





procedure TForm1.btn_fire_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ fire button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_fire_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ left button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ right button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: DOWN                                                               }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_power_down  else                              // Power On/Off
  if Key = aKeys[2].word then Inp_select_down else                              // Select
  if Key = aKeys[3].word then Inp_start_down  else                              // Start

  if Key = aKeys[4].word then Inp_left_down   else                              // Left
  if Key = aKeys[5].word then Inp_right_down  else                              // Right
  if Key = aKeys[6].word then Inp_fire_down                                     // Fire
{DEBUG} //else if Key = $39 then Vid_Display_All_Sprites(true)                  // Debug: press '9' to display all sprites
  ;
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: UP                                                                 }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[2].word then Inp_select_up else                                // Select
  if Key = aKeys[3].word then Inp_start_up  else                                // Start

  if Key = aKeys[4].word then Inp_left_up   else                                // Left
  if Key = aKeys[5].word then Inp_right_up  else                                // Right
  if Key = aKeys[6].word then Inp_fire_up;                                      // Fire
end;











{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.inp_power_down;
{ power switch: PRESS                                                          }
begin
  if iMode = 0 then                                                             // tabletop off ->
    begin
      btn_power_on.Visible := true;
      im_background_screen_on.Visible := true;
      iMode := 1; Gam_Set_Mode;                                                   //            -> demo (1/2)
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_ON; // "click to turn off"
    end
  else                                                                          // any mode ->
    begin
      btn_power_on.Visible := false;
      im_background_screen_on.Visible := false;
      iMode := 0; Gam_Set_Mode;                                                   //        -> tabletop off
      btn_power_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_PWR_OFF; // "click to turn on"
    end;
end;





procedure TForm1.inp_select_down;
{ select button: PRESS                                                         }
begin
  btn_select_down.Visible := true;

  if iMode = 2 then                                                             // demo (2/2)
    begin
      Snd_Play(pfs_selecbtn);
      Inc(iSkill);

      if iSkill = 5 then
        iSkill := 1;

      Vid_Display_Panel_Skill(iSkill, true);
    end;
end;





procedure TForm1.inp_select_up;
{ select button: RELEASE                                                       }
begin
  btn_select_down.Visible := false;
end;





procedure TForm1.inp_start_down;
{ start button: PRESS                                                          }
begin
  btn_start_down.Visible := true;

  if iMode = 2 then                                                             // demo (2/2) ->
    begin                                                                         //          -> game
      iMode := 3; Gam_Set_Mode;
   end;
end;





procedure TForm1.inp_start_up;
{ start button: RELEASE                                                        }
begin
  btn_start_down.Visible := false;
end;





procedure TForm1.inp_left_down;
{ joystick left: PRESS                                                         }
  var
    i, j: integer;
begin
  if (not bJoyPressed) then
    begin
      bJoyPressed := true;
      btn_joystick_placeholder.Picture := btn_joystick_left.Picture;
    end;

  if bControllers then
    if (iPhase <= 3) then
      begin                                                                     // phases 1, 2 and 3
        if iShipX > 1 then
          begin
            Vid_Display_Ship(false);
            Dec(iShipX);
            Vid_Display_Ship(true);

            bControllers := false;
            Timer_KeyPress.Enabled := false;
            Timer_KeyPress.Enabled := true;
          end;

        if aIm_Enemy[iShipX, 1].Visible or Odd(aFire[iShipX, 2]) then             // checks collisions
          Gam_Phase123_Miss;                                                        // a miss has occured
      end
    else                                                                        // phase 4
      if iShipX > 1 then
        begin
          Vid_Display_Shuttle(false);
          Dec(iShipX);
          Vid_Display_Shuttle(true);

          bControllers := false;
          Timer_KeyPress.Enabled := false;
          Timer_KeyPress.Enabled := true;

          for i := 1 to 5 do
            for j := 2 to 6 do
              aIm_Fire[i, j].Visible := false;
        end;
end;





procedure TForm1.inp_left_up;
{ joystick left: RELEASE                                                       }
begin
  bJoyPressed := false;
  btn_joystick_placeholder.Picture := nil;
end;





procedure TForm1.inp_right_down;
{ joystick right: PRESS                                                        }
  var
    i, j: integer;
begin
  if (not bJoyPressed) then
    begin
      bJoyPressed := true;
      btn_joystick_placeholder.Picture := btn_joystick_right.Picture;
    end;

  if bControllers then
    if (iPhase <= 3) then
      begin                                                                     // phases 1, 2 and 3
        if iShipX < 5 then
          begin
            Vid_Display_Ship(false);
            Inc(iShipX);
            Vid_Display_Ship(true);

            bControllers := false;
            Timer_KeyPress.Enabled := false;
            Timer_KeyPress.Enabled := true;
          end;

        if aIm_Enemy[iShipX, 1].Visible or Odd(aFire[iShipX, 2]) then             // checks collisions
          Gam_Phase123_Miss;                                                        // a miss has occured
      end
    else                                                                        // phase 4
      if iShipX < 5 then
        begin
          Vid_Display_Shuttle(false);
          Inc(iShipX);
          Vid_Display_Shuttle(true);

          bControllers := false;
          Timer_KeyPress.Enabled := false;
          Timer_KeyPress.Enabled := true;

          for i := 1 to 5 do
            for j := 2 to 6 do
              aIm_Fire[i, j].Visible := false;
        end;
end;





procedure TForm1.inp_right_up;
{ joystick right: RELEASE                                                      }
begin
  bJoyPressed := false;
  btn_joystick_placeholder.Picture := nil;
end;





procedure TForm1.inp_fire_down;
{ fire button: PRESS                                                           }
begin
  if (not bFirePressed) then
    begin
      bFirePressed := true;
      btn_fire_down.Visible := true;
    end;

  if bControllers then
    if (iPhase <= 3) then                                                       // phases 1, 2 and 3
      if not bBulletOnScreen then                                                 // if no player's ship bullet on screen
        begin
          iBulletX := iShipX;                                                       // sets column where bullet is shot

          Inc(aFire[iBulletX, 2], 2);                                               // places bullet in numeric array
          aIm_Fire[iBulletX, 2].Visible := true;                                    // displays bullet
          Snd_Play(pfs_fire);                                                       // plays "fire"

          bBulletOnScreen := true;                                                  // bullet appears on screen now

          Timer_Phase123_Player_Fire.Interval := 80;
          Timer_Phase123_Player_Fire.Enabled := true;
//          Debug_aEnemy;
//          Debug_aFire;
          Gam_Phase123_Check_Collision;                                             // checks for collision (attacking enemy ship in row 2)
        end
      else
    else                                                                        // phase 4
      if not bPhase4SndPlay then                                                  // if "thrust" sound effect is not currently playing
        begin
          Snd_Play(pfs_thrust);                                                     // plays "thrust"
          bPhase4SndPlay := true;

          Timer_Phase4_Shuttle_Down.Enabled := false;                               // shuttle stops going down

          Timer_Phase4_Shuttle_Thrust.Interval := 1500;                             // shuttle starts going up
          Timer_Phase4_Shuttle_Thrust.Enabled := true;

          iLoopDisplayThrust := 1;                                                  // displays thrust animation
          Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
          Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
        end;
end;





procedure TForm1.inp_fire_up;
{ fire button: RELEASE                                                         }
  var
    i, j: integer;
begin
  bFirePressed := false;
  btn_fire_down.Visible := false;

  if (iPhase = 4) and (Timer_Phase4_Dock.Enabled) then
    begin
      Snd_Silence;
      bPhase4SndPlay := false;

      Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
      Timer_Phase4_Shuttle_Thrust.Enabled := false;
      Timer_Phase4_Shuttle_Down.Enabled := true;

      for i := 1 to 5 do
        for j := 2 to 6 do
          aIm_Fire[i, j].Visible := false;
    end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Demo_1.Enabled                        := false;
  Timer_Demo_2.Enabled                        := false;
  Timer_Game_Start.Enabled                    := false;
  Timer_Game_Over.Enabled                     := false;

  Timer_KeyPress.Enabled                      := false;

  Timer_Phase123_Miss.Enabled                 := false;
  Timer_Phase123_Player_Fire.Enabled          := false;
  Timer_Phase123_Enemy_Fire.Enabled           := false;

  Timer_Phase1_Enemy_Row8.Enabled             := false;
  Timer_Phase1_Enemy_Row7.Enabled             := false;
  Timer_Phase1_Enemy_Attacking.Enabled        := false;
  Timer_Phase1_Enemy_Rows78_Hit.Enabled       := false;
  Timer_Phase1_Enemy_Attacking_Hit.Enabled    := false;
  Timer_Phase1_Reprise.Enabled                := false;
  Timer_Phase1_Complete.Enabled               := false;

  Timer_Phase2_Enemy_Attacking.Enabled        := false;
  Timer_Phase2_Enemy_Attacking_Hit.Enabled    := false;
  Timer_Phase2_Reprise.Enabled                := false;
  Timer_Phase2_Complete.Enabled               := false;

  Timer_Phase3_Mothership.Enabled             := false;
  Timer_Phase3_Mothership_Hit.Enabled         := false;
  Timer_Phase3_Reprise.Enabled                := false;
  Timer_Phase3_Complete.Enabled               := false;

  Timer_Phase4_Shuttle_Thrust.Enabled         := false;
  Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
  Timer_Phase4_Dock.Enabled                   := false;
  Timer_Phase4_Shuttle_Down.Enabled           := false;
  Timer_Phase4_Miss_Dock.Enabled              := false;
  Timer_Phase4_Bonus.Enabled                  := false;
end;





procedure TForm1.Gam_Reset_Numeric_Arrays;
{ resets all aEnemy and aFire array cells to zero                              }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    begin
      for j := 1 to 9 do
        aEnemy[i, j] := 0;

      for j := 2 to 9 do
        aFire[i, j] := 0;
    end;

  // clears coordinates of attacking enemy ships
  for i := 1 to 2 do
   begin
     aEnemy6[i].iPosX := 0;
     aEnemy6[i].iPosY := 0;
     aEnemy6[i].iDirection := 0;
   end;

  bBulletOnScreen := false;
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
  var
    i, j: integer;
begin
  case iMode of
    0: begin                                                                    // tabletop off
         Gam_Stop_All_Timers;                                                           // stops action
         Vid_Display_All_Sprites(false);                                                // hides all sprites
         Gam_Reset_Numeric_Arrays;                                                      // resets aEnemy and aFire arrays
         Snd_Silence;                                                                   // stops playing sounds

         bControllers := false;                                                         // disables controller
       end;
    1: begin                                                                    // demo (1/2)
         Gam_Stop_All_Timers;                                                           // stops action
         Gam_Reset_Numeric_Arrays;                                                      // resets aEnemy and aFire arrays
         Gam_Phase123_Set_Sprites;                                                      // sets enemy ships sprites to "whole"
         Vid_Display_All_Sprites(false);                                                // hides all sprites
         Snd_Silence;                                                                   // stops playing sounds

         bControllers := false;                                                         // disables controller

         iLoop := 1;                                                                    // demo mode starts
         Timer_Demo_1.Interval := 1;
         Timer_Demo_1.Enabled := true;
       end;
    2: begin                                                                    // demo (2/2)
       end;                                                                             // does nothing
    3: begin                                                                    // game
         Gam_Stop_All_Timers;                                                           // stops action
         Vid_Display_All_Sprites(false);                                                // hides all sprites
         Gam_Reset_Numeric_Arrays;                                                      // resets aEnemy and aFire arrays
         Gam_Phase123_Set_Sprites;                                                      // sets enemy ships sprites to "whole"
         Snd_Silence;                                                                   // stops playing sounds

         iLoop := 1;                                                                    // game starts
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;
       end;
  end; // case..of
end;





procedure TForm1.Gam_Phase123_Set_Sprites;
{ phases 1, 2 and 3: sets all enemy and player's sprites to "whole"            }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    for j := 1 to 6 do
      aIm_Enemy[i, j].Picture := aIm_Enemy_Picture[i, j].Whole.Picture;

  for i := 1 to 5 do
    aIm_Ship[i].Picture := aIm_Ship_Picture[i].Whole.Picture;
end;





procedure TForm1.Gam_Phase123_Check_Collision;
{ phases 1, 2 and 3: checks whether player's missile hit an enemy ship         }
  var
    i, j: integer;
begin
  case iPhase of
    1: for j := 2 to 8 do                                                       // row 9 is not calculated
         for i := 1 to 5 do
           if (aFire[i, j] > 1) and (aEnemy[i, j] > 0) then                       // collision here
             begin
               Gam_Stop_All_Timers;
               Snd_Silence;
               bControllers := false;

               iEnemyHitX := i;
               iEnemyHitY := j;

               if aEnemy[i, j] = 1 then
                 begin                                                              // common enemy ship was hit
                   Snd_Play(pfs_hit);

                   Timer_Phase1_Enemy_Rows78_Hit.Interval := 80;
                   Timer_Phase1_Enemy_Rows78_Hit.Enabled := true;
                 end
               else
                 begin                                                              // moving enemy ship was hit
                   iLoopHit := 1;
                   Timer_Phase1_Enemy_Attacking_Hit.Interval := 100;
                   Timer_Phase1_Enemy_Attacking_Hit.Enabled := true;
                 end;
             end;
    2: for j := 2 to 8 do                                                       // row 9 is not calculated
         for i := 1 to 5 do
           if (aFire[i, j] > 1) and (aEnemy[i, j] > 0) then                       // collision here
             begin
               Gam_Stop_All_Timers;
               Snd_Silence;
               bControllers := false;

               iEnemyHitX := i;
               iEnemyHitY := j;
               iLoopHit := 1;
               Timer_Phase2_Enemy_Attacking_Hit.Interval := 100;
               Timer_Phase2_Enemy_Attacking_Hit.Enabled := true;
             end;
    3: for i := 1 to 5 do
         if (aFire[i, 9] > 1) and (aEnemy[i, 9] > 0) then                         // collision here
           begin
             aIm_Fire[i, 9].Visible := true;
             Gam_Stop_All_Timers;
             Snd_Silence;
             bControllers := false;

             iEnemyHitX := i;
             iLoopHit := 1;
             Timer_Phase3_Mothership_Hit.Interval := 100;
             Timer_Phase3_Mothership_Hit.Enabled := true;
           end;
  end; // case..of
end;





procedure TForm1.Gam_Phase123_Miss;
{ phases 1, 2 and 3: a miss has occured                                        }
begin
  Gam_Stop_All_Timers;
  Snd_Silence;
  bControllers := false;

  iLoop := 1;
  Timer_Phase123_Miss.Interval := 1;
  Timer_Phase123_Miss.Enabled := true;
end;





procedure TForm1.Gam_Phase4_Set_Sprites;
{ phase 4: sets enemy ships sprites to "shuttle" and player's ship sprites to  }
{ dock                                                                         }
  var
    i, j: integer;
begin
  for i := 1 to 5 do
    begin
      for j := 1 to 6 do
        aIm_Enemy[i, j].Picture := aIm_Enemy_Picture[i, j].Body.Picture;

      aIm_Ship[i].Picture := aIm_Ship_Picture[i].Dock.Picture;
    end;
end;





procedure TForm1.Gam_Phase4_Check_Dock;
{ phase 4: checks whether shuttle docked correctly or not                      }
begin
  iLoop := 1;

  if (iShipX = iDockX) and (iShipY = 1) then                                    // shuttle docked
    begin                                                                         // bonus routine
      Timer_Phase4_Bonus.Interval := 1;
      Timer_Phase4_Bonus.Enabled := true;
    end
  else
    begin                                                                         // no bonus routine
      Timer_Phase4_Miss_Dock.Interval := 1;
      Timer_Phase4_Miss_Dock.Enabled := true;
    end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DEBUGGER                                                     * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Debug_aEnemy;
{ displays contents of the aEnemy array on screen                              }
  var
    i, j: integer;
begin
  La_debug_aEnemy.caption := '';

  for j := 9 downto 1 do
    begin
      for i := 1 to 5 do
        La_debug_aEnemy.caption := La_debug_aEnemy.caption + inttostr(aEnemy[i, j]);

      La_debug_aEnemy.caption := La_debug_aEnemy.caption + ' ';
    end;
end;




procedure TForm1.Debug_aFire;
{ displays contents of the aFire array on screen                               }
  var
    i, j: integer;
begin
  La_debug_aFire.caption := '';

  for j := 9 downto 2 do
    begin
      for i := 1 to 5 do
        La_debug_aFire.caption := La_debug_aFire.caption + inttostr(aFire[i, j]);

      La_debug_aFire.caption := La_debug_aFire.caption + ' ';
   end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: NON-INGAME                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Demo_1Timer(Sender: TObject);
{ plays "gameon" tune and goes into "select" mode (demo 2/2)                   }
  var
    i: integer;
begin
  case iLoop of
    1: begin
         // plays "start" tune
         Snd_Play(pfs_gameon);
         Timer_Demo_1.Interval := 3300;
       end;
    2: begin
         // sets mode to demo (2/2) + displays final score
         iSkill := 1; Vid_Display_Panel_Skill(iSkill, true);

         iMode := 2; Gam_Set_Mode;

         iEnemiesLeft := MAX_ENEMIES; { needed to render sprites by the Timer_Phase1_Enemy7 and _Enemy8 routines }

         aIm_Enemy[5, 8].Visible := true; aEnemy[5, 8] := 1; Timer_Phase1_Enemy_Row8.Interval := 200; Timer_Phase1_Enemy_Row8.Enabled := true;
         aIm_Enemy[1, 7].Visible := true; aEnemy[1, 7] := 1; Timer_Phase1_Enemy_Row7.Interval := 350; Timer_Phase1_Enemy_Row7.Enabled := true;

         Timer_Demo_1.Interval := 800;
       end;
    3: begin
         // displays motherships then makes Timer_Demo_2 start
         for i := 2 to 4 do
           begin
             aEnemy[i, 9] := 1;
             aIm_Enemy[i, 9].Visible := true;
           end;

         iLoop := 0; { becomes 1 with next "Inc" }
         Timer_Demo_2.Interval := aGameSpeed[iSkill];
         Timer_Demo_2.Enabled := true;

         Timer_Demo_1.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Demo_2Timer(Sender: TObject);
{ enemy ships move in demonstration mode; player can select skill level        }
  var
    i, j: integer;
begin
  if iLoop <= 26 then
    Timer_Demo_2.Interval := aGameSpeed[iSkill]                                 // normal speed till the 3rd attacking enemy ship appears
  else
    Timer_Demo_2.Interval := aGameSpeed[iSkill] - 80;                           // faster speed since the 3rd attacking enemy ship appears

  for j := 1 to 6 do
    for i := 1 to 5 do
      aEnemy[i, j] := 0;                                                        // hides all enemy ships sprites temporary

  for j := 7 to 8 do
    for i := 1 to 5 do
      if aEnemy[i, j] >= 2 then
        Dec(aEnemy[i, j], 2);                                                   // clears attacking enemy ship when it's in rows 7 or 8

  case iLoop of
     // first attacking enemy ship
     1:       aEnemy[4, 6] := 2;                                                // appears
     2:       aEnemy[3, 6] := 2;                                                // left
     3:       aEnemy[4, 6] := 2;                                                // right
     4:       aEnemy[5, 6] := 2;                                                // right
     5: begin aEnemy[5, 5] := 2; aFire[5, 5] := 1; aIm_Fire[5, 5].Visible := true; Timer_Phase123_Enemy_Fire.Interval := 150; Timer_Phase123_Enemy_Fire.Enabled := true; end; // down + fire
     6: begin aEnemy[4, 5] := 2; aFire[4, 5] := 1; aIm_Fire[4, 5].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // left + fire
     7:       aEnemy[5, 5] := 2;                                                // right
     8:       aEnemy[4, 5] := 2;                                                // left
     9: begin aEnemy[3, 5] := 2; aFire[3, 5] := 1; aIm_Fire[3, 5].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // left + fire
    10:       aEnemy[3, 4] := 2;                                                // down
    11:       aEnemy[3, 3] := 2;                                                // down
    12:       aEnemy[2, 3] := 2;                                                // left
    13: begin aEnemy[3, 3] := 2; aFire[3, 3] := 1; aIm_Fire[3, 3].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // right + fire
    14:       aEnemy[2, 3] := 2;                                                // left
    15:       aEnemy[1, 3] := 2;                                                // left
    16:       aEnemy[2, 3] := 2;                                                // right
    17:       aEnemy[2, 2] := 2;                                                // down
    18:       aEnemy[2, 1] := 2;                                                // down
    19:       aEnemy[1, 1] := 2;                                                // disappears
    // second attacking enemy ship
    20:       aEnemy[3, 7] := 2;                                                // appears
    21:       aEnemy[4, 6] := 2;                                                // down + right
    22:       aEnemy[5, 5] := 2;                                                // down + right
    23:       aEnemy[4, 4] := 2;                                                // down + left
    24: begin aEnemy[5, 3] := 2; aFire[5, 3] := 1; aIm_Fire[5, 3].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // down + right + fire
    25:       aEnemy[4, 2] := 2;                                                // down + left
    26:       aEnemy[3, 1] := 2;                                                // disappears
    // third attacking enemy ship
    27:       aEnemy[2, 6] := 2;                                                // appears
    28:       aEnemy[1, 6] := 2;                                                //  2 left
    29:       aEnemy[2, 7] := 2;                                                //  3 up + right
    30:       aEnemy[3, 8] := 2;                                                //  4 up + right
    31:       aEnemy[4, 8] := 2;                                                //  5 right
    32:       aEnemy[5, 7] := 2;                                                //  6 down + right
    33:       aEnemy[5, 6] := 2;                                                //  7 down
    34:       aEnemy[4, 6] := 2;                                                //  8 left
    35:       aEnemy[3, 7] := 2;                                                //  9 up + left
    36:       aEnemy[3, 8] := 2;                                                // 10 up
    37: begin aEnemy[4, 7] := 2; aFire[4, 7] := 1; aIm_Fire[4, 7].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 11 down + right + fire
    38:       aEnemy[5, 7] := 2;                                                // 12 right
    39:       aEnemy[4, 6] := 2;                                                // 13 down + left
    40:       aEnemy[3, 5] := 2;                                                // 14 down + left
    41:       aEnemy[2, 5] := 2;                                                // 15 left
    42:       aEnemy[1, 6] := 2;                                                // 16 up + left
    43:       aEnemy[1, 7] := 2;                                                // 17 up
    44:       aEnemy[2, 7] := 2;                                                // 18 right
    45:       aEnemy[3, 7] := 2;                                                // 19 right
    46:       aEnemy[3, 6] := 2;                                                // 20 down
    47:       aEnemy[2, 6] := 2;                                                // 21 left
    48: begin aEnemy[1, 7] := 2; aFire[1, 7] := 1; aIm_Fire[1, 7].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; //22 up + left + fire
    49:       aEnemy[2, 7] := 2;                                                // 23 right
    50:       aEnemy[3, 7] := 2;                                                // 24 right
    51: begin aEnemy[3, 6] := 2; aFire[3, 6] := 1; aIm_Fire[3, 6].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 25 down + fire
    52:       aEnemy[3, 5] := 2;                                                // 26 left
    53: begin aEnemy[2, 5] := 2; aFire[2, 5] := 1; aIm_Fire[2, 5].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 27 left + fire
    54:       aEnemy[3, 4] := 2;                                                // 28 down + right
    55:       aEnemy[3, 3] := 2;                                                // 29 down
    56:       aEnemy[2, 3] := 2;                                                // 30 left
    57:       aEnemy[1, 4] := 2;                                                // 31 up + right
    58: begin aEnemy[1, 5] := 2; aFire[1, 5] := 1; aIm_Fire[1, 5].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 32 up + fire
    59: begin aEnemy[2, 6] := 2; aFire[2, 6] := 1; aIm_Fire[2, 6].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 33 up + right + fire
    60:       aEnemy[3, 5] := 2;                                                // 34 down + right
    61: begin aEnemy[3, 4] := 2; aFire[3, 4] := 1; aIm_Fire[3, 4].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 35 down + fire
    62:       aEnemy[2, 4] := 2;                                                // 36 left
    63:       aEnemy[1, 5] := 2;                                                // 37 up + left
    64:       aEnemy[2, 4] := 2;                                                // 38 down + right
    65:       aEnemy[1, 5] := 2;                                                // 39 up + left
    66:       aEnemy[1, 6] := 2;                                                // 40 up
    67:       aEnemy[2, 7] := 2;                                                // 41 up + right
    68: begin aEnemy[3, 6] := 2; aFire[3, 6] := 1; aIm_Fire[3, 6].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 42 down + right + fire
    69:       aEnemy[3, 5] := 2;                                                // 43 down
    70: begin aEnemy[2, 5] := 2; aFire[2, 5] := 1; aIm_Fire[2, 5].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 44 left + fire
    71:       aEnemy[1, 6] := 2;                                                // 45 up + left
    72:       aEnemy[1, 7] := 2;                                                // 46 up
    73:       aEnemy[2, 8] := 2;                                                // 47 up + right
    74:       aEnemy[2, 7] := 2;                                                // 48 down
    75: begin aEnemy[1, 8] := 2; aFire[1, 8] := 1; aIm_Fire[1, 8].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; end; // 49 up + left + fire
    76:       aEnemy[2, 8] := 2;                                                // 50 left
    77:       aEnemy[3, 7] := 2;                                                // 51 down + right
    78:       aEnemy[4, 7] := 2;                                                // 52 right
    79:       aEnemy[5, 6] := 2;                                                // 53 down + right
    80:       aEnemy[5, 5] := 2;                                                // 54 down
    81:       aEnemy[4, 5] := 2;                                                // 55 left
    82:       aEnemy[3, 6] := 2;                                                // 56 up + left
    83:       aEnemy[3, 7] := 2;                                                // 57 up
    84:       aEnemy[4, 8] := 2;                                                // 58 up + right
    85:       aEnemy[5, 8] := 2;                                                // 59 right
    86:       aEnemy[5, 7] := 2;                                                // 60 down
    87:       aEnemy[4, 6] := 2;                                                // 61 down + left
    88:       aEnemy[3, 5] := 2;                                                // 62 down + left
    89:       aEnemy[2, 5] := 2;                                                // 63 left
    90:       aEnemy[1, 6] := 2;                                                // 64 up + left
    91:       aEnemy[2, 7] := 2;                                                // 65 up + right
    92:       aEnemy[3, 8] := 2;                                                // 66 up + right
    93:       aEnemy[4, 7] := 2;                                                // 67 down + right
    94:       aEnemy[3, 6] := 2;                                                // 68 down + left
    95: begin aEnemy[2, 6] := 2; aFire[2, 6] := 1; aIm_Fire[2, 6].Visible := true; Timer_Phase123_Enemy_Fire.Enabled := true; // 69 left + fire
          iLoop := 26;                                                          // becomes 27 with next "Inc"
        end;
  end; // case..of

  // displays enemies
  for j := 1 to 6 do
   for i := 1 to 5 do
     aIm_Enemy[i, j].Visible := aEnemy[i, j] <> 0;

  Snd_Play(pfs_tick);

  Inc(iLoop);
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ plays tune, sets phase 1 and then game begins                                }
  var
    i: integer;
begin
  case iLoop of
    1: begin
         // plays "startbtn" sound
         Snd_Play(pfs_startbtn);

         Timer_Game_Start.Interval := 100;
       end;
    2: begin
         // displays lives left + player's ship in the middle of the bottom row
         iLives := 5; Vid_Display_Panel_Lives(iLives, true);
         iShipX := 3; Vid_Display_Ship(true);

         Timer_Game_Start.Interval := 1450;
       end;
    3: begin
         // turns off display plays, and plays "start" tune
         Vid_Display_All_Sprites(false);
         Snd_Play(pfs_start);

         Timer_Game_Start.Interval := 1650;
       end;
    4: begin
         // sets variables for phase 1, 2 and 3
         iPhase := 1;
         iEnemiesLeft := MAX_ENEMIES;
         Timer_KeyPress.Interval := PHASE123_KEYPRESS;

         // sets variables for game start
         iScore := 0; Vid_Display_Panel_Score(iScore, true);
         iShipX := 3; Vid_Display_Ship(true);
         bControllers := true;

         // resets attacking enemy ships infos (coordinates, direction)
         for i := 1 to 2 do
           begin
             aEnemy6[i].iPosX := 0;
             aEnemy6[i].iPosY := 0;
             aEnemy6[i].iDirection := 0;
           end;

         // sets timers speed
         Timer_Phase123_Enemy_Fire.Interval := 250;
         Timer_Phase1_Enemy_Attacking.Interval := aGameSpeed[iSkill];
         Timer_Phase1_Enemy_Row7.Interval := 350;
         Timer_Phase1_Enemy_Row8.Interval := 200;

         // enemy ships in rows 7 and 8 appear
         Timer_Phase1_Enemy_Row7.Enabled := true;
         Timer_Phase1_Enemy_Row8.Enabled := true;

         Timer_Game_Start.Interval := 800;
       end;
    5: // motherships appear in row 9
       for i := 2 to 4 do
         begin
           aEnemy[i, 9] := 1;
           aIm_Enemy[i, 9].Visible := true;
         end;
    6: begin
         // attacking enemy ship(s) appear(s)
         Timer_Phase1_Enemy_Attacking.Enabled := true;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ plays "game over" tune, displays final score, then sets mode to demo (2/2)   }
  var
    i: integer;
begin
  case iLoop of
    1: begin
         // plays "game over" tune
         Snd_Play(pfs_gameover);

         Timer_Game_Over.Interval := 5800;
       end;
    2: begin
         // sets mode to demo (2/2) + displays final score
         iMode := 2; Gam_Set_Mode;

         Vid_Display_Panel_Score(iScore, true);

         iEnemiesLeft := MAX_ENEMIES; { needed to render sprites by the Timer_Phase1_Enemy7 and _Enemy8 routines }

         aIm_Enemy[5, 8].Visible := true; aEnemy[5, 8] := 1; Timer_Phase1_Enemy_Row8.Interval := 200; Timer_Phase1_Enemy_Row8.Enabled := true;
         aIm_Enemy[1, 7].Visible := true; aEnemy[1, 7] := 1; Timer_Phase1_Enemy_Row7.Interval := 350; Timer_Phase1_Enemy_Row7.Enabled := true;

         Timer_Game_Over.Interval := 800;
       end;
    3: begin
         // displays motherships then makes Timer_Demo_2 start
         for i := 2 to 4 do
           begin
             aEnemy[i, 9] := 1;
             aIm_Enemy[i, 9].Visible := true;
           end;

         iLoop := 0; { becomes 1 with next "Inc" }
         Timer_Demo_2.Interval := aGameSpeed[iSkill];
         Timer_Demo_2.Enabled := true;

         Timer_Game_Over.Enabled := false;
    end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN ALL PHASES                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_KeypressTimer(Sender: TObject);
{ enables disabled controller at every xxx milliseconds                        }
begin
  bControllers := true;
  Timer_KeyPress.Enabled := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASES 1, 2 AND 3                            * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase123_Player_FireTimer(Sender: TObject);
{ phases 1, 2 and 3: player ship bullet                                        }
  var
    j: integer;
    b: boolean;
begin
  // bullet in row 9: clear
  if aFire[iBulletX, 9] > 1 then
    begin
      Dec(aFire[iBulletX, 9], 2);
      aIm_Fire[iBulletX, 9].Visible := false;
    end;

  // bullet in row 2 to 8: move
  for j := 8 downto 2 do
    if aFire[iBulletX, j] > 1 then
      begin
        Dec(aFire[iBulletX, j    ], 2); aIm_Fire[iBulletX, j    ].Visible := false;
        Inc(aFire[iBulletX, j + 1], 2); aIm_Fire[iBulletX, j + 1].Visible := true;
      end;

  // disables timer in case there's no more bullets on screen
  b := false;

  for j := 2 to 9 do
    if aFire[iBulletX, j] > 1 then
      b := true;

  bBulletOnScreen := b;

  Timer_Phase123_Player_Fire.Enabled := b;

  // checks collision between player's ship bullet and enemy ships
  Gam_Phase123_Check_Collision;
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase123_Enemy_FireTimer(Sender: TObject);
{ phases 1, 2 and 3: enemy ships bullets                                       }
  var
    i, j: integer;
    b:    boolean;
begin
  if (iMode = 3) and Odd(aFire[iShipX, 2]) then                                 // if ingame and bullet hit player's ship
    Gam_Phase123_Miss                                                             // a miss has occured
  else                                                                          // else
    begin                                                                         // bullets move
      for i := 1 to 5 do
        begin
          // bullets in row 2: clear
          if Odd(aFire[i, 2]) then
            Dec(aFire[i, 2]);

          // bullets in rows 3 to 9: move
          for j := 3 to 9 do
            if Odd(aFire[i, j]) then
              begin
                Dec(aFire[i, j]);
                Inc(aFire[i, j - 1]);
              end;

          // renders bullets
          for j := 2 to 9 do
            aIm_Fire[i, j].Visible := aFire[i, j] > 0;
        end;

      // disables timer in case there's no more bullets on screen
      b := false;

      for j := 2 to 9 do
        for i := 1 to 5 do
          if aIm_Fire[i, j].Visible then
            b := true;

      Timer_Phase123_Enemy_Fire.Enabled := b;
    end;
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase123_MissTimer(Sender: TObject);
{ phases 1, 2 and 3: a miss has occured                                        }
  var
    i: integer;
begin
  case iLoop of
    1: Timer_Phase123_Miss.Interval := 70;
    2: begin
         // hides player's ship and displays "enemy ship wings"
         Vid_Display_Ship(false);

         for i := 1 to 5 do
           begin
             aIm_Enemy[i, 1].Picture := aIm_Enemy_Picture[i, 1].Wings.Picture;
             aIm_Enemy[iShipX, 1].Visible := true;
             aIm_Fire[i, 2].Visible := false;
           end;
       end;
    3: begin Vid_Display_Ship(true);  aIm_Enemy[iShipX, 1].Visible := false; end; // player's ship
    4: begin Vid_Display_Ship(false); aIm_Enemy[iShipX, 1].Visible := true;  end; // enemy ship wings
    5: begin Vid_Display_Ship(true);  aIm_Enemy[iShipX, 1].Visible := false; end; // player's ship
    6: begin Vid_Display_Ship(false); aIm_Enemy[iShipX, 1].Visible := true;  end; // enemy ship wings
    7: begin Vid_Display_Ship(true);  aIm_Enemy[iShipX, 1].Visible := false; end; // player's ship
    8: begin Vid_Display_Ship(false); aIm_Enemy[iShipX, 1].Visible := true;       // enemy ship wings
         Snd_Play(pfs_miss);
         Timer_Phase123_Miss.Interval := 250;
       end;
    9: begin
         // hides all sprites, resets sprites pictures, resets numeric arrays to 0)
         Vid_Display_All_Sprites(false);
         Gam_Reset_Numeric_Arrays;
         Gam_Phase123_Set_Sprites;

         Timer_Phase123_Miss.Interval := 1500;
//         Debug_aEnemy;
//         Debug_aFire;
       end;
   10: begin
         iLoop := 0; { becomes 1 with next "Inc" }

         Dec(iLives);

         if iLives > 0 then                                                     // if any lives left
           case iPhase of                                                         // game reprises
             1: begin Timer_Phase1_Reprise.Interval := 1; Timer_Phase1_Reprise.Enabled := true; end;
             2: begin Timer_Phase2_Reprise.Interval := 1; Timer_Phase2_Reprise.Enabled := true; end;
             3: begin Timer_Phase3_Reprise.Interval := 1; Timer_Phase3_Reprise.Enabled := true; end;
           end // case..of
         else                                                                   // else
           begin                                                                  // game is over
             Timer_Game_Over.Interval := 1;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Phase123_Miss.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 1 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase1_Enemy_Row8Timer(Sender: TObject);
{ phase 1: makes enemy ships in row 8 appear and move                          }
  var
    i: integer;
    b: boolean;
begin
  // clears enemy ship in (1, 8)
  if Odd(aEnemy[1, 8]) then
    Dec(aEnemy[1, 8]);

  // moves the remaining enemy ships left
  for i := 1 to 4 do
    if Odd(aEnemy[i + 1, 8]) then
      begin
        Dec(aEnemy[i + 1, 8]);
        Inc(aEnemy[i, 8]);
      end;

  // displays enemy ships in row 8
  for i := 1 to 5 do
    aIm_Enemy[i, 8].Visible := aEnemy[i, 8] > 0;

  // generates new enemy ship in (5, 8)
  b := true;

  for i := 1 to 5 do                                                            // do not generate if there's already enemy ships
    if Odd(aEnemy[i, 8]) then
      b := false;

  if Odd(aEnemy[2, 8]) then                                                     // generates in case there's an enemy ship in (2, 8)
    b := true;

  if iEnemiesLeft < 3 then                                                      // do not generate in case there's still 3 attacking enemy ships to go
    b:=false;

  if b then                                                                     // actual generation
    begin
      Inc(aEnemy[5, 8]);
      aIm_Enemy[5, 8].Visible := true;
    end;

  Gam_Phase123_Check_Collision;                                                 // checks collision with player's ship bullet
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase1_Enemy_Row7Timer(Sender: TObject);
{ phase 1: makes enemy ships in row 7 appear and move                          }
  var
    i: integer;
    b: boolean;
begin
  // clears enemy ship in (1, 8)
  if Odd(aEnemy[5, 7]) then
    Dec(aEnemy[5, 7]);

  // moves the remaining enemy ships right
  for i := 5 downto 2 do
    if Odd(aEnemy[i - 1, 7]) then
      begin
        Dec(aEnemy[i - 1, 7]);
        Inc(aEnemy[i, 7]);
      end;

  // displays enemy ships in row 7
  for i := 1 to 5 do
    aIm_Enemy[i, 7].Visible := aEnemy[i, 7] > 0;

  // generates new enemy ship in (5, 8)
  b := true;

  for i := 1 to 5 do                                                            // do not generate if there's already enemy ships
    if Odd(aEnemy[i, 7]) then
      b := false;

  if Odd(aEnemy[4, 7]) then                                                     // generates in case there's an enemy ship in (4, 7)
    b := true;

  if iEnemiesLeft < 3 then                                                      // do not generate in case there's still 3 attacking enemy ships to go,
    b := false;

  if b then                                                                     // actual generation
    begin
      Inc(aEnemy[1, 7]);
      aIm_Enemy[1, 7].Visible := true;
    end;

  Gam_Phase123_Check_Collision;                                                 // checks collision with player's ship bullet
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase1_Enemy_AttackingTimer(Sender: TObject);
{ phase 1: makes attacking enemy ships appear and move                         }
  var
    i, j: integer;
begin
  // clears cells with value below zero (prevents from bug)
  for j := 1 to 9 do
    for i := 1 to 5 do
      if aEnemy[i, j] < 0 then
        aEnemy[i, j] := 0;

 // attacking enemy ships: generates/moves/fires
  for i := 1 to (iSkill + 1) div 2 do                                           // do once or twice (once if iSkill=1-2, twice if iSkill=3/4)
    if (aEnemy6[i].iPosX = 0) and (aEnemy6[i].iPosY = 0) then                     // if there's no attacking enemy ship on screen
      begin                                                                         // generate new attacking enemy(s)
        aEnemy6[i].iDirection := Random(2) + 1;                                       // dir   is  1 (horz) or  2 (diag)
        aEnemy6[i].iValue     := 30 + (aEnemy6[i].iDirection * 10);                   // value is 40 (horz) or 50 (diag)
        aEnemy6[i].iPosX      := Random(5) + 1;                                       // x     is random column 1-5
        aEnemy6[i].iPosY      := 6 + aEnemy6[i].iDirection;                           // y     is  7 (horz) or  8 (diag)

        Inc(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                       // updates cell value +2/+4
        aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;                // displays attacking enemy ship
      end
    else                                                                          // there's attacking enemy on screen
      begin                                                                         // moves attacking enemy(s), depending on its direction
        case aEnemy6[i].iDirection of
          1: begin                                                                    // 1) *** HORIZONTAL ***
               Dec(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                  // updates numeric array (dec)

               if aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY] = 0 then                   // if nothing on that certain cell
                 aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := false;          // hides attacking enemy

               // attacking enemy ship(s) moves
               case Random(3)                                                           // goes left, right or down
                 of 0: begin Dec(aEnemy6[i].iPosX); if aEnemy6[i].iPosX < 1 then aEnemy6[i].iPosX := 2; end; // goes left  (or right if leftmost)
                    1: begin Inc(aEnemy6[i].iPosX); if aEnemy6[i].iPosX > 5 then aEnemy6[i].iPosX := 4; end; // goes right (or left if rightmost)
                    2:       Dec(aEnemy6[i].iPosY);                                                          // goes down
                 end; // case..of
             end;
          2: begin                                                                    // 2) *** DIAGONAL ***
               Dec(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                  // updates numeric array (dec)

               if aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY] = 0 then                   // if nothing on that certain cell
                 aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := false;          // hides attacking enemy

               // attacking enemy ship(s) moves
               case Random(2) of
                 0: begin Dec(aEnemy6[i].iPosX); if aEnemy6[i].iPosX < 1 then aEnemy6[i].iPosX := 2; end; // goes left
                 1: begin Inc(aEnemy6[i].iPosX); if aEnemy6[i].iPosX > 5 then aEnemy6[i].iPosX := 4; end; // goes right
               end; // case..of

               Dec(aEnemy6[i].iPosY);                                                                    // + goes down
             end;
        end; // case..of

        if aEnemy6[i].iPosY < 1 then                                                // if it's at the bottom of the screen
          begin
            aEnemy6[i].iPosX := 0;                                                    // gets ready to generate a new attacking enemy ship
            aEnemy6[i].iPosY := 0;                                                    // gets ready to generate a new attacking enemy ship
            aEnemy6[i].iDirection := 0;                                               // gets ready to generate a new attacking enemy ship
          end
        else                                                                        // if it's in 2nd to 8th row
          begin
            Inc(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                   // updates numeric array
            aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;            // renders attacking enemy ship

            // attacking enemy ship(s) fires
            if (aEnemy6[i].iPosY > 1)                                                 // if it's on 2nd to 8th row
            and ((Random(10) = 0)                                                     // if random 1 out of 10
            and (not Odd(aFire[aEnemy6[i].iPosX, aEnemy6[i].iPosY]))) then            // if no bullet already in that cell
              begin
                Inc(aFire[aEnemy6[i].iPosX, aEnemy6[i].iPosY]);                         // updates numeric array (inc)
                aIm_Fire[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;           // renders bullet
                Timer_Phase123_Enemy_Fire.Enabled := true;                              // starts bullet movement timer
              end;

            if (aEnemy6[i].iPosY = 1) and (aEnemy6[i].iPosX = iShipX) then            // checks collisions
              Gam_Phase123_Miss;                                                        // a miss has occured

            Gam_Phase123_Check_Collision;                                             // checks collisions
          end;
      end;

  Snd_Play(pfs_tick);                                                           // plays "tick"
end;





procedure TForm1.Timer_Phase1_Enemy_Rows78_HitTimer(Sender: TObject);
{ phase 1: enemy ships in rows 7 and 8 was hit: increases score, hides enemy   }
{ ship sprite, and then action reprises                                        }
  var
    tmpScore: integer;
begin
  // updates numeric arrays
  aEnemy[iEnemyHitX, iEnemyHitY] := 0;                                          // removes enemy from numeric array
  aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := false;                           // hides sprite

  Dec(aFire[iEnemyHitX, iEnemyHitY], 2);                                        // removes bullet

  if aFire[iEnemyHitX, iEnemyHitY] = 0                                          // hides bullet in case there's player's bullet only on that cell
    then aIm_Fire[iEnemyHitX, iEnemyHitY].Visible := false;

  tmpScore := 10 + (iEnemyHitY div 8) * 10;                                     // sets score = 10 (row 7) or 20 (row 8)

  Inc(iScore, tmpScore); Vid_Display_Panel_Score(iScore, true);                 // increases score

  // game reprises
  Timer_Phase1_Enemy_Row8.Enabled      := true;
  Timer_Phase1_Enemy_Row7.Enabled      := true;
  Timer_Phase1_Enemy_Attacking.Enabled := true;
  Timer_Phase123_Enemy_Fire.Enabled    := true;

  bBulletOnScreen := false;                                                     // player's bullet not on screen anymore
  bControllers := true;

  Timer_Phase1_Enemy_Rows78_Hit.Enabled := false;
end;





procedure TForm1.Timer_Phase1_Enemy_Attacking_HitTimer(Sender: TObject);
{ phase 1: attacking enemy ships was hit: increases score, hides enemy ship    }
{ sprite, and then action reprises                                             }
  var
    i, tmpScore, tmpHit: integer;
begin
  // enemy ship sprite blinks
  if Odd(iLoopHit) then
    if iEnemyHitY > 6 then
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := false                        // hides sprite if row = 7 or 8
    else
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Body.Picture   // displays body if row <= 6
  else
    if iEnemyHitY > 6 then
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := true                         // displays sprite if row = 7 or 8
    else
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Wings.Picture; // displays wings if row <= 6

  // updates arrays, etc.
  case iLoopHit of
    1: begin                                                                    // removes bullet
         Dec(aFire[iEnemyHitX, iEnemyHitY], 2);

         if aFire[iEnemyHitX, iEnemyHitY] = 0 then
           aIm_Fire[iEnemyHitX, iEnemyHitY].Visible := false;
       end;
    7: begin                                                                    // adds score, removes enemy(s) ship, etc.
         tmpScore := 0;
         tmpHit   := 0;

         if Odd(aEnemy[iEnemyHitX, iEnemyHitY]) then                            // if hit enemy is in row 7 or 8
           begin                                                                  // saves attacking enemy ships in that same cell, and kills the "normal" one
             Dec(aEnemy[iEnemyHitX, iEnemyHitY]);                                   // kills enemy
             tmpScore := 10 + (iEnemyHitY div 8) * 10;                              // sets score = 10 (row 7) or 20 (row 8)
           end
         else                                                                   // else hit enemy is in row <= 6
           for i := 1 to (iSkill + 1) div 2 do                                    // do once or twice (once if iSkill=1-2, twice if iSkill=3/4)
             if  (aEnemy6[i].iPosX = iEnemyHitX)
             and (aEnemy6[i].iPosY = iEnemyHitY) then
               begin
                 // updates numeric array
                 Dec(aEnemy[iEnemyHitX, iEnemyHitY], i * 2);

                 // gets ready to generate a new attacking enemy
                 aEnemy6[i].iPosX := 0;
                 aEnemy6[i].iPosY := 0;
                 aEnemy6[i].iDirection := 0;

                 // gets ready to add points to score
                 Inc(tmpScore, 40 + (aEnemy6[i].iDirection div 2) * 10);

                 // gets ready to decrease "enemies-to-go"
                 Inc(tmpHit);
               end;

         Dec(iEnemiesLeft, tmpHit);                                               // decreases "enemies-to-go"
         Inc(iScore, tmpScore); Vid_Display_Panel_Score(iScore, true);            // adds points to score
         Snd_Play(pfs_hit);                                                       // plays sound sample

         aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := false;                      // hides killed enemy ship

         // stage is complete if "enemies-to-go" below zero
         // note: 0 is no good, because iEnemiesLeft might go -1
         // in case of destroying 2 enemies at once when 1 only
         // is "to go"
         if iEnemiesLeft < 1 then
           begin
             Gam_Stop_All_Timers;                                               // stops all timers, incl the current one!
             bControllers := false;

             iLoop := 0; { becomes 1 with next "Inc" }
             Timer_Phase1_Complete.Interval := 100;
             Timer_Phase1_Complete.Enabled := true;
           end;
        end;
    8: begin                                                                    // game reprises
         if (iEnemyHitY > 0) and (iEnemyHitY < 7) then
           aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Whole.Picture;

         // action reprises
         Timer_Phase1_Enemy_Row8.Enabled      := true;
         Timer_Phase1_Enemy_Row7.Enabled      := true;
         Timer_Phase1_Enemy_Attacking.Enabled := true;
         Timer_Phase123_Enemy_Fire.Enabled    := true;
         bBulletOnScreen := false;                                              // player's bullet not on screen anymore
         bControllers := true;

         Timer_Phase1_Enemy_Attacking_Hit.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopHit);
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase1_RepriseTimer(Sender: TObject);
{ phase 1: game reprises after player's ship was destroyed                     }
  var
    i: integer;
begin
  case iLoop of
    1: begin                                                                    // displays lives left
         iShipX := 3; Vid_Display_Ship(true);
         Vid_Display_Panel_Lives(iLives, true);

         for i := 2 to 4 do
           begin
             aEnemy[i, 9] := 1;
             aIm_Enemy[i, 9].Visible := true;
           end;

         Timer_Phase1_Reprise.Interval := 1500;
       end;
    2: begin                                                                    // hides display and plays "start" tune
         Gam_Reset_Numeric_Arrays;
         Vid_Display_All_Sprites(false);
         Snd_Play(pfs_start);

         Timer_Phase1_Reprise.Interval := 1650;
       end;
    3: begin                                                                    // action reprises
         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);
         Timer_Keypress.Interval := PHASE123_KEYPRESS;

         bControllers := true;

         Timer_Phase1_Enemy_Row8.Interval := 200;
         Timer_Phase1_Enemy_Row7.Interval := 350;
         Timer_Phase1_Enemy_Attacking.Interval := aGameSpeed[iSkill]; { must be here }

         Timer_Phase1_Enemy_Row7.Enabled := true;
         Timer_Phase1_Enemy_Row8.Enabled := true;

         Timer_Phase1_Reprise.Interval := 800;
       end;
    4: for i := 2 to 4 do                                                       // displays motherships
         begin
           aEnemy[i, 9] := 1;
           aIm_Enemy[i, 9].Visible := true;
         end;
    5: begin                                                                    // gets ready to generate new attacking enenmy ships
         for i := 1 to 2 do
           begin
             aEnemy6[i].iPosX := 0;
             aEnemy6[i].iPosY := 0;
             aEnemy6[i].iDirection := 0;
           end;

         Timer_Phase1_Enemy_Attacking.Enabled := true;                            // new attacking enemy ships appear

         Timer_Phase1_Reprise.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Phase1_CompleteTimer(Sender: TObject);
{ phase 1: phase complete -> goes into phase 2                                 }
  var
    i: integer;
begin
  case iLoop of
    1: begin                                                                    // hides all sprites, resets arrays, plays "phase" sound
         Vid_Display_All_Sprites(false);
         Gam_Reset_Numeric_Arrays;
         Gam_Phase123_Set_Sprites;
         Snd_Play(pfs_phase);

         Timer_Phase1_Complete.Interval := 1800;
       end;
    2: begin                                                                    // phase 2 begins
         for i := 2 to 4 do
           begin
             aEnemy[i, 9] := 1;
             aIm_Enemy[i, 9].Visible := true;
           end;

         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);
         bControllers := true;
         bBulletOnScreen := false;                                              // player's bullet not on screen anymore

         iPhase := 2;
         iEnemiesLeft := MAX_ENEMIES;                                           // sets "enemies-to-go" to maximum possible

         for i := 1 to 2 do                                                     // gets ready to generate new attacking enenmy ships
           begin
             aEnemy6[i].iPosX := 0;
             aEnemy6[i].iPosY := 0;
             aEnemy6[i].iDirection := 0;
           end;

         Timer_Phase2_Enemy_Attacking.Interval := aGameSpeed[iSkill] - 50;      // starts action
         Timer_Phase2_Enemy_Attacking.Enabled := true;

         Timer_Phase1_Complete.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 2 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase2_Enemy_AttackingTimer(Sender: TObject);
{ phase 2: makes attacking enemy ships appear and move                         }
  var
    i, j: integer;
begin
  // clears cells with value below zero (prevents from bug)
  for j := 1 to 9 do
    for i := 1 to 5 do
      if aEnemy[i, j] < 0 then
        begin
          aEnemy[i, j] := 0;
          aIm_Enemy[i, j].Visible := false;
        end;

 // attacking enemy ships: generates/moves/fires
  for i := 1 to (iSkill + 1) div 2 do                                           // do once or twice (once if iSkill=1-2, twice if iSkill=3/4)
    if (aEnemy6[i].iPosX = 0) and (aEnemy6[i].iPosY = 0) then                     // if there's no attacking enemy ship on screen
      begin                                                                         // generate new attacking enemy(s)
        aEnemy6[i].iDirection := Random(8) + 1;                                       // 1=N; 2=NE; 3=E; 4=SE; 5=S; 6=SW; 7=W; 8=NW
        aEnemy6[i].iValue     := 70;                                                  // value is 70
        aEnemy6[i].iPosX      := Random(5) + 1;                                       // x     is random column 1-5
        aEnemy6[i].iPosY      := Random(2) + 7;                                       // y     is random row 7-8

        Inc(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                       // updates cell value +2/+4
        aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;                // displays attacking enemy ship
      end
    else
      begin                                                                         // moves attacking enemy(s), depending on its direction
        if random(10) > 0 then                                                        // 1 chance out of 10
          Inc(aEnemy6[i].iDirection, Random(2))                                         // change direction: increase by 1 or 2
        else                                                                          // 9 chances out of 10
          Inc(aEnemy6[i].iDirection, Random(2) + 4);                                    // change direction: increase by 4 or 5

        if aEnemy6[i].iDirection > 8 then                                             // fixes direction value if above 8
          Dec(aEnemy6[i].iDirection, 8);

        Dec(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                       // gets ready to move attacking enemy ship

        if aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY] = 0 then                        // hides sprite if none on that certain cell
          aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := false;

        // attacking enemy ship(s) moves
        case aEnemy6[i].iDirection of
          1: begin                                                                    // EAST
               Inc(aEnemy6[i].iPosX);

               if aEnemy6[i].iPosX > 5 then
                 begin
                   aEnemy6[i].iPosX := 5;
                   Dec(aEnemy6[i].iPosY);
                   aEnemy6[i].iDirection := 3;
                 end;
             end;
          2: begin                                                                    // SOUTH-EAST
               Inc(aEnemy6[i].iPosX);

               if aEnemy6[i].iPosX > 5 then
                 aEnemy6[i].iPosX := 5;

               Dec(aEnemy6[i].iPosY);
               aEnemy6[i].iDirection := 3;
             end;
           3: Dec(aEnemy6[i].iPosY);                                                  // SOUTH
           4: begin                                                                   // SOUTH-WEST
                Dec(aEnemy6[i].iPosX);

                if aEnemy6[i].iPosX < 1 then
                  begin
                    aEnemy6[i].iPosX := 1;
                    Inc(aEnemy6[i].iPosY);

                    if aEnemy6[i].iPosY > 8 then
                      begin
                        aEnemy6[i].iPosY := 8;
                        Inc(aEnemy6[i].iPosX);
                        aEnemy6[i].iDirection := 1;
                      end;
                  end
                else
                  Dec(aEnemy6[i].iPosY);
              end;
           5: begin                                                                   // WEST
                Dec(aEnemy6[i].iPosX);

                if aEnemy6[i].iPosX < 1 then
                  begin
                    aEnemy6[i].iPosX := 1;
                    Inc(aEnemy6[i].iPosY);
                    aEnemy6[i].iDirection := 7;

                    if aEnemy6[i].iPosY > 8 then
                      begin
                        aEnemy6[i].iPosY := 8;
                        Inc(aEnemy6[i].iPosX);
                        aEnemy6[i].iDirection := 1;
                      end;
                  end;
              end;
           6: begin                                                                   // NORTH-WEST
                Dec(aEnemy6[i].iPosX);
                if aEnemy6[i].iPosX < 1 then
                  begin aEnemy6[i].iPosX := 1;
                    Inc(aEnemy6[i].iPosY);
                    aEnemy6[i].iDirection := 7;

                    if aEnemy6[i].iPosY > 8 then
                      begin
                        aEnemy6[i].iPosY := 8;
                        Inc(aEnemy6[i].iPosX);
                        aEnemy6[i].iDirection:=1;
                      end;
                  end
                else
                  begin
                    Inc(aEnemy6[i].iPosY);

                    if aEnemy6[i].iPosY > 8 then
                      begin
                        aEnemy6[i].iPosY := 8;
                        Inc(aEnemy6[i].iPosX, 2);
                        aEnemy6[i].iDirection := 1;

                        if aEnemy6[i].iPosX > 5 then
                          begin
                            aEnemy6[i].iPosX := 5;
                            Dec(aEnemy6[i].iPosY);
                            aEnemy6[i].iDirection := 3;
                          end;
                      end;
                  end;
              end;
           7: begin                                                                   // NORTH
                Inc(aEnemy6[i].iPosY);

                if aEnemy6[i].iPosY > 8 then
                  begin
                    aEnemy6[i].iPosY := 8;
                    Inc(aEnemy6[i].iPosX);
                    aEnemy6[i].iDirection := 1;

                    if aEnemy6[i].iPosX > 5 then
                      begin
                        aEnemy6[i].iPosX := 5;
                        Dec(aEnemy6[i].iPosY);
                        aEnemy6[i].iDirection := 3;
                      end
                  end;
              end;
           8: begin                                                                   // NORTH-EAST
                Inc(aEnemy6[i].iPosX);

                if aEnemy6[i].iPosX > 5 then
                  begin
                    aEnemy6[i].iPosX := 5;
                    Dec(aEnemy6[i].iPosY);
                    aEnemy6[i].iDirection := 3;
                  end
                else
                  begin
                    Inc(aEnemy6[i].iPosY);

                    if aEnemy6[i].iPosY > 8 then
                      begin
                        aEnemy6[i].iPosY := 8;
                        aEnemy6[i].iDirection := 1;
                      end;
                  end;
              end;
        end; // case..of

        if aEnemy6[i].iPosY < 1 then                                                  // if it's at the bottom of the screen
          begin
            aEnemy6[i].iPosX := 0;                                                      // gets ready to generate a new attacking enemy ship
            aEnemy6[i].iPosY := 0;                                                      // gets ready to generate a new attacking enemy ship
            aEnemy6[i].iDirection := 0;                                                 // gets ready to generate a new attacking enemy ship
          end;

        if  (aEnemy6[i].iPosY > 0)                                                    // if attacking enemy ship is still on screen
        and (aEnemy6[i].iPosY < 9)
        and (aEnemy6[i].iPosX > 0)
        and (aEnemy6[i].iPosX < 6) then
          begin
            Inc(aEnemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY], i * 2);                     // updates numeric array
            aIm_Enemy[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;              // renders attacking enemy ship

            // attacking enemy ship(s) fires
            if (aEnemy6[i].iPosY > 1)                                                 // if it's on 2nd to 8th row
            and ((Random(6) = 0)                                                      // if random 1 out of 6
            and (not Odd(aFire[aEnemy6[i].iPosX, aEnemy6[i].iPosY]))) then            // if no bullet already in that cell
              begin
                Inc(aFire[aEnemy6[i].iPosX, aEnemy6[i].iPosY]);                         // updates numeric array (inc)
                aIm_Fire[aEnemy6[i].iPosX, aEnemy6[i].iPosY].Visible := true;           // renders bullet
                Timer_Phase123_Enemy_Fire.Enabled := true;                              // starts bullet movement timer
              end;

            if (aEnemy6[i].iPosY = 1) and (aEnemy6[i].iPosX = iShipX) then            // checks collisions
              Gam_Phase123_Miss;                                                        // a miss has occured

            Gam_Phase123_Check_Collision;                                             // checks collisions
          end;
      end;
      
  Snd_Play(pfs_tick);
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase2_Enemy_Attacking_HitTimer(Sender: TObject);
{ phase 2: attacking enemy ships was hit: increases score, hides enemy ship    }
{ sprite, and then action reprises                                             }
  var
    i, tmpScore, tmpHit: integer;
begin
  // enemy ship sprite blinks
  case iLoopHit of
    1, 3, 5: if iEnemyHitY > 6 then
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := false                        // hides sprite if row = 7 or 8
    else
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Body.Picture;   // displays body if row <= 6
    2, 4, 6: if iEnemyHitY > 6 then
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := true                         // displays sprite if row = 7 or 8
    else
      aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Wings.Picture; // displays wings if row <= 6
  end; // case..of

  // updates arrays, etc.
  case iLoopHit of
    1: begin                                                                    // removes bullet
         Dec(aFire[iEnemyHitX, iEnemyHitY], 2);

         if aFire[iEnemyHitX, iEnemyHitY] = 0 then
           aIm_Fire[iEnemyHitX, iEnemyHitY].Visible := false;
       end;
    7: begin                                                                    // adds score, removes enemy(s) ship, etc.
         tmpScore := 0;
         tmpHit   := 0;

         for i := 1 to (iSkill + 1) div 2 do                                    // do once or twice (once if iSkill=1-2, twice if iSkill=3/4)
           if  (aEnemy6[i].iPosX = iEnemyHitX)
           and (aEnemy6[i].iPosY = iEnemyHitY) then
             begin
               // updates numeric array
               Dec(aEnemy[iEnemyHitX, iEnemyHitY], i * 2);

               // gets ready to generate a new attacking enemy
               aEnemy6[i].iPosX := 0;
               aEnemy6[i].iPosY := 0;
               aEnemy6[i].iDirection := 0;

               // gets ready to add points to score
               Inc(tmpScore, 70);

               // gets ready to decrease "enemies-to-go"
               Inc(tmpHit);
             end;

         Dec(iEnemiesLeft, tmpHit);                                             // decreases "enemies-to-go"
         Inc(iScore, tmpScore); Vid_Display_Panel_Score(iScore, true);          // adds points to score
         Snd_Play(pfs_hit);                                                     // plays sound sample

         aIm_Enemy[iEnemyHitX, iEnemyHitY].Visible := false;                    // hides killed enemy ship

         // stage is complete if "enemies-to-go" below zero
         // note: 0 is no good, because iEnemiesLeft might go -1
         // in case of destroying 2 enemies at once when 1 only
         // is "to go"
         if iEnemiesLeft < 1 then
           begin
             Gam_Stop_All_Timers;                                                 // stops all timers, incl the current one!
             bControllers := false;

             iLoop := 0; { becomes 1 with next "Inc" }
             Timer_Phase2_Complete.Interval := 100;
             Timer_Phase2_Complete.Enabled := true;
           end;
       end;
    8: begin                                                                    // game reprises
         if (iEnemyHitY > 0) and (iEnemyHitY < 7) then
           aIm_Enemy[iEnemyHitX, iEnemyHitY].Picture := aIm_Enemy_Picture[iEnemyHitX, iEnemyHitY].Whole.Picture;

         // action reprises
         Timer_Phase2_Enemy_Attacking.Enabled := true;
         Timer_Phase123_Enemy_Fire.Enabled    := true;
         bBulletOnScreen := false;                                              // player's bullet not on screen anymore
         bControllers := true;

         Timer_Phase2_Enemy_Attacking_Hit.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopHit);
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase2_RepriseTimer(Sender: TObject);
{ phase 2: game reprises after player's ship was destroyed                     }
  var
    i: integer;
begin
  case iLoop of
    1: begin                                                                    // displays lives left
         iShipX := 3; Vid_Display_Ship(true);
         Vid_Display_Panel_Lives(iLives, true);

        for i := 2 to 4 do
          begin
            aEnemy[i, 9] := 1;
            aIm_Enemy[i, 9].Visible := true;
          end;

         Timer_Phase2_Reprise.Interval := 1500;
       end;
    2: begin                                                                    // hides display and plays "start" tune
         Gam_Reset_Numeric_Arrays;
         Vid_Display_All_Sprites(false);
         Snd_Play(pfs_start);

         Timer_Phase2_Reprise.Interval := 1650;
       end;
    3: begin                                                                    // action reprises
         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);
         Timer_Keypress.Interval := PHASE123_KEYPRESS;

         bControllers := true;

         Timer_Phase1_Enemy_Attacking.Interval := aGameSpeed[iSkill] - 50; { must be here }

         Timer_Phase2_Reprise.Interval := 800;
       end;
    4: for i := 2 to 4 do                                                       // displays motherships
         begin
           aEnemy[i, 9] := 1;
           aIm_Enemy[i, 9].Visible := true;
         end;
    5: begin                                                                    // gets ready to generate new attacking enenmy ships
         for i := 1 to 2 do
           begin
             aEnemy6[i].iPosX := 0;
             aEnemy6[i].iPosY := 0;
             aEnemy6[i].iDirection := 0;
           end;

         Timer_Phase2_Enemy_Attacking.Enabled := true;                            // new attacking enemy ships appear

         Timer_Phase2_Reprise.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Phase2_CompleteTimer(Sender: TObject);
{ phase 2: game reprises after player's ship was destroyed                     }
  var
    i: integer;
begin
  case iLoop of
    1: begin                                                                    // hides all sprites, resets arrays, plays "phase" sound
         Vid_Display_All_Sprites(false);
         Gam_Reset_Numeric_Arrays;
         Gam_Phase123_Set_Sprites;
         Snd_Play(pfs_phase);

         Timer_Phase2_Complete.Interval := 1800;
       end;
    2: begin                                                                    // phase 2 begins
         for i := 2 to 4 do
           begin
             aEnemy[i, 9] := 1;
             aIm_Enemy[i, 9].Visible := true;
           end;

         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);
         bControllers := true;
         bBulletOnScreen := false;                                              // player's bullet not on screen anymore

         iPhase := 3;
         iEnemiesLeft := 3;                                                     // sets "enemies-to-go" to 3 (amount of motherships)

         Timer_Phase3_Mothership.Interval := aGameSpeed[iSkill];                // starts action
         Timer_Phase3_Mothership.Enabled := true;

         Timer_Phase2_Complete.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 3 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase3_MothershipTimer(Sender: TObject);
{ phase 3: makes motherships appear and move                                   }
  var
    i, tmpRnd: integer;
    b:         boolean;
begin
  // clears cells in row 9
  for i := 1 to 5 do
    begin
      aEnemy[i, 9] := 0;
      aIm_Enemy[i, 9].Visible := false;
    end;

  for i := 1 to iEnemiesLeft do
    begin
      b := false;

      repeat
        tmpRnd := Random(5) + 1;

        if aEnemy[tmpRnd, 9] = 0 then
          begin
            b := true;                                                          // ends repeat..until
            aEnemy[tmpRnd, 9] := 1;
            aIm_Enemy[tmpRnd, 9].Visible := true;

            // mothership(s) fires
            if (Random(4) = 0) then
              begin
                Inc(aFire[tmpRnd, 9]);
                aIm_Fire[tmpRnd, 9].Visible := true;
                Timer_Phase123_Enemy_Fire.Enabled := true;
              end
          end
      until
        b;
    end;

  for i := 1 to 5 do
    aEnemy9[i] := aEnemy[i, 9];

  Gam_Phase123_Check_Collision;
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase3_Mothership_HitTimer(Sender: TObject);
{ phase 3: mothership was hit: increases score, hides mothership sprite, and   }
{ then action reprises                                                         }
  var
    i, j: integer;
begin
  // enemy ship sprite blinks
  aIm_Enemy[iEnemyHitX, 9].Visible := not Odd(iLoopHit);


  // updates arrays, etc.
  case iLoopHit of
    1: begin                                                                    // removes bullet
         for j := 2 to 9 do
           for i := 1 to 5 do
             begin
               if aFire[i, j] > 1 then dec(aFire[i, j], 2);
               if aFire[i, j] < 1 then aFire[i, j] := 0;
               if aFire[i, j] = 0 then aIm_Fire[i, j].Visible := false;
               if aFire[i, j] = 1 then aIm_Fire[i, j].Visible := true;
              end;
         end;
    7: begin                                                                    // adds score, removes enemy(s) ship, etc.
         Inc(iScore, 100); Vid_Display_Panel_Score(iScore, true);
         Dec(iEnemiesLeft);
         Snd_Play(pfs_hit);

         // stage is complete if "enemies-to-go" below zero
         // note: 0 is no good, because iEnemiesLeft might go -1
         // in case of destroying 2 enemies at once when 1 only
         // is "to go"
         if iEnemiesLeft < 1 then
           begin
             Gam_Stop_All_Timers;                                                 // stops all timers, incl the current one!
             bControllers := false;

             iLoop := 0; { becomes 1 with next "Inc" }
             Timer_Phase3_Complete.Interval := 100;
             Timer_Phase3_Complete.Enabled := true;
           end;
       end;
    8: begin                                                                    // game reprises
         aEnemy[iEnemyHitX, 9] := 0;
         aIm_Enemy[iEnemyHitX, 9].Visible := false;

         // action reprises
         Timer_Phase3_Mothership.Enabled   := true;
         Timer_Phase123_Enemy_Fire.Enabled := true;
         bBulletOnScreen := false;                                                 // player's bullet not on screen anymore
         bControllers := true;

         Timer_Phase3_Mothership_Hit.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopHit);
//  Debug_aEnemy;
//  Debug_aFire;
end;





procedure TForm1.Timer_Phase3_RepriseTimer(Sender: TObject);
{ phase 3: game reprises after player's ship was destroyed                     }
  var
    i: integer;
begin
  case iLoop of
    1: begin                                                                    // displays lives left
         iShipX := 3; Vid_Display_Ship(true);
         Vid_Display_Panel_Lives(iLives, true);

         for i := 1 to 5 do
           aIm_Enemy[i, 9].Visible := aEnemy9[i] = 1;

         Timer_Phase3_Reprise.Interval := 1500;
       end;
    2: begin                                                                    // hides display and plays "start" tune
         Gam_Reset_Numeric_Arrays;
         Vid_Display_All_Sprites(false);
         Snd_Play(pfs_start);

         Timer_Phase3_Reprise.Interval := 1650;
       end;
    3: begin                                                                    // action reprises
         for i := 1 to 5 do
           if aEnemy9[i] = 1 then
             begin
               aEnemy[i, 9] := 1;
               aIm_Enemy[i, 9].Visible := true;
             end
           else
             begin
               aEnemy[i, 9] := 0;
               aIm_Enemy[i, 9].Visible := false;
             end;

         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);

         bControllers := true;

         Timer_Phase3_Mothership.Interval := aGameSpeed[iSkill] - 50; { must be here }
         Timer_Phase3_Mothership.Enabled := true;

         Timer_Phase3_Reprise.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Phase3_CompleteTimer(Sender: TObject);
{ phase 3: game reprises after player's ship was destroyed                     }
  var
    i, j: integer;
begin
  case iLoop of
    1: begin                                                                    // hides all sprites, resets arrays, plays "phase" sound
         Vid_Display_All_Sprites(false);
         Gam_Reset_Numeric_Arrays;
         Gam_Phase123_Set_Sprites;
         Snd_Play(pfs_phase);

         Timer_Phase3_Complete.Interval := 3300;
       end;
    2: begin                                                                    // phase 3 begins: shuttle takes off
         Vid_Display_Panel_Score(iScore, true);
         Vid_Display_Ship(true);                                                  // shuttle takes off from the same X where player's ship is
         Snd_Play(pfs_thrust);

         Timer_Phase3_Complete.Interval := 1300;
       end;
    3: begin                                                                    // shuttle taking off: animation 1
         // sets sprites array pictures
         for i := 1 to 5 do
           begin
             aIm_Ship[i].Picture := aIm_Ship_Picture[i].Dock.Picture;
             aIm_Enemy[i, 1].Picture := aIm_Enemy_Picture[i, 1].Body.Picture;
            end;

         for i := 1 to 5 do
           for j := 2 to 6 do
            aIm_Enemy[i, j].Picture := aIm_Enemy_Picture[i, j].Body.Picture;

         // "good" on score panel
         aIm_panel_digit_pos[1].Picture := im_panel_letter_g.Picture;
         aIm_panel_digit_pos[2].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[3].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[4].Picture := im_panel_letter_d.Picture;

         // shuttle movement
         iShipY := 1;                                                           // (iShipY = 1)
         Vid_Display_Shuttle(false); Inc(iShipY); Vid_Display_Shuttle(true);

         // thrust smoke animation
         iLoopDisplayThrust := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
       end;
    4: begin                                                                    // shuttle taking off: animation 2
         // score on score panel
         Vid_Display_Panel_Score(iScore, true);
         aIm_Fire[iShipX,iShipY].Visible := false;
         Vid_Display_Shuttle(false); Inc(iShipY); Vid_Display_Shuttle(true);    // (iShipY = 2)

         // thrust smoke animation
         iLoopDisplayThrust := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
       end;
    5: begin                                                                    // shuttle taking off: animation 3
         // "good" on score panel
         aIm_panel_digit_pos[1].Picture := im_panel_letter_g.Picture;
         aIm_panel_digit_pos[2].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[3].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[4].Picture := im_panel_letter_d.Picture;

         // shuttle movement
         aIm_Fire[iShipX,iShipY].Visible := false;
         Vid_Display_Shuttle(false); Inc(iShipY); Vid_Display_Shuttle(true);    // (iShipY = 3)

         // thrust smoke animation
         iLoopDisplayThrust := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
       end;
    6: begin                                                                    // shuttle taking off: animation 4
         // score on score panel
         Vid_Display_Panel_Score(iScore, true);
         aIm_Fire[iShipX,iShipY].Visible := false;
         Vid_Display_Shuttle(false); Inc(iShipY); Vid_Display_Shuttle(true);    // (iShipY = 4)

         // thrust smoke animation
         iLoopDisplayThrust := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
       end;
    7: begin                                                                    // shuttle taking off: animation 5
         // "good" on score panel
         aIm_panel_digit_pos[1].Picture := im_panel_letter_g.Picture;
         aIm_panel_digit_pos[2].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[3].Picture := im_panel_letter_o.Picture;
         aIm_panel_digit_pos[4].Picture := im_panel_letter_d.Picture;

         // shuttle movement
         aIm_Fire[iShipX, iShipY].Visible := false;
         Vid_Display_Shuttle(false); Inc(iShipY); Vid_Display_Shuttle(true);    // (iShipY = 5)

         // thrust smoke animation
         iLoopDisplayThrust := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Interval := 1;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := true;
       end;
    8: begin                                                                    // shuttle taking off: animation ends
         // score decreases, shuttle descends
         iPhase := 4;

         aIm_Fire[iShipX, iShipY].Visible := false;
         Timer_Phase3_Complete.Enabled := false;
         Timer_Phase4_Shuttle_Thrust_Display.Enabled := false;

         Snd_Silence;
         bPhase4SndPlay := false;
         iBonus := 999;
         Vid_Display_Panel_Score(iBonus, true);
         iDockX := iShipX;

         Timer_Phase4_Shuttle_Down.Interval := 2260;
         Timer_Phase4_Shuttle_Down.Enabled := true;

         iLoopDock := 1;
         Timer_Phase4_Dock.Interval := 16;
         Timer_Phase4_Dock.Enabled := true;

         Timer_KeyPress.Interval := PHASE4_KEYPRESS;
         bControllers := true;
       end;
  end; // case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS: USED IN PHASE 4 ONLY                                 * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Phase4_Shuttle_ThrustTimer(Sender: TObject);
{ phase 4: thrust smoke array management                                       }
  var
    i, j: integer;
begin
  if bFirePressed and (iShipY < 6) then
    begin
      for i := 1 to 5 do
        for j := 2 to 6 do
          aIm_Fire[i, j].Visible := false;

      Vid_Display_Shuttle(false);
      Inc(iShipY);
      Vid_Display_Shuttle(true);
    end
end;





procedure TForm1.Timer_Phase4_Shuttle_Thrust_DisplayTimer(Sender: TObject);
{ phase 4: thrust smoke animation                                              }
  var
    i, j: integer;
begin
  if Odd(iLoopDisplayThrust) and
    (iShipX >= 1) and (iShipX <= 5) and
    (iShipY >= 2) and (iShipY <= 6) then
    aIm_Fire[iShipX, iShipY].Visible := true
  else
    for i := 1 to 5 do
      for j := 2 to 6 do
        aIm_Fire[i, j].Visible := false;

  Inc(iLoopDisplayThrust);

  if iLoopDisplayThrust = 3 then
    iLoopDisplayThrust := 1;

  Timer_Phase4_Shuttle_Thrust_Display.Interval := 162;
end;





procedure TForm1.Timer_Phase4_Shuttle_DownTimer(Sender: TObject);
{ phase 4: shuttle descends                                                    }
begin
  if iShipY > 2 then                                                            // if shuttle's Y coordinate above 1
    begin                                                                         // decreases coordinate
      Vid_Display_Shuttle(false);
      Dec(iShipY);
      Vid_Display_Shuttle(true);
    end
  else                                                                          // else
    begin                                                                         // checks shuttle and dock positions -> bonus or miss?
      Gam_Stop_All_Timers;
      Snd_Silence;
      bControllers := false;

      Vid_Display_Shuttle(false);
      Dec(iShipY);
      Vid_Display_Shuttle(true);

      Gam_Phase4_Check_Dock;
    end;
end;





procedure TForm1.Timer_Phase4_DockTimer(Sender: TObject);
{ phase 4: dock moves left to right and back                                   }
begin
  if iBonus > 0 then                                                            // if bonus score above 0
    begin                                                                         // moves dock
      Dec(iBonus, 1); Vid_Display_Panel_Score(iBonus, true);

      if iLoopDock = 35 then
        begin
          Vid_Display_Dock(false);

          Inc(iDockX);

          if iDockX > 5 then
            iDockX := 1;

          Vid_Display_Dock(true);

          iLoopDock := 0;
        end;

      Inc(iLoopDock);
    end
  else                                                                          // else
    begin                                                                         // checks shuttle and dock positions -> bonus or miss?
      Gam_Stop_All_Timers;
      Snd_Silence;
      bControllers := false;

      Gam_Phase4_Check_Dock;
    end;
end;





procedure TForm1.Timer_Phase4_Miss_DockTimer(Sender: TObject);
{ phase 4: shuttle missed dock                                                 }
begin
  case iLoop of
    1: Timer_Phase4_Miss_Dock.Interval := 1000;
    2: begin
         Snd_Play(pfs_missdock);
         Timer_Phase4_Miss_Dock.Interval := 2000;
       end;
    3: begin
         Vid_Display_Dock(false); Vid_Display_Shuttle(false);
         Gam_Phase123_Set_Sprites;
         iPhase := 1;
         iEnemiesLeft := MAX_ENEMIES;

         iLoop := 0; { becomes 1 with next "Inc" }
         Timer_Phase1_Reprise.Interval := 1;
         Timer_Phase1_Reprise.Enabled := true;

         Timer_Phase4_Miss_Dock.Enabled := false;
       end;
  end; // case..of

  Inc(iLoop);
end;





procedure TForm1.Timer_Phase4_BonusTimer(Sender: TObject);
{ phase 4: shuttle matched dock                                                }
begin
  case iLoop of
    1: begin
         Snd_Play(pfs_bonus);
         Timer_Phase4_Bonus.Interval := 5000;
       end;
    2: begin
         Inc(iScore, iBonus);

         // hides dock and shuttle
         Vid_Display_Dock(false);
         Vid_Display_Shuttle(false);

         // gets ready to start a new "phase 1"
         Gam_Phase123_Set_Sprites;
         iPhase := 1;
         iEnemiesLeft := MAX_ENEMIES;

         // phase 1 starts
         iLoop := 0; { becomes 1 with next "Inc" }
         Timer_Phase1_Reprise.Interval := 1;
         Timer_Phase1_Reprise.Enabled := true;

         Timer_Phase4_Bonus.Enabled := false;
       end;
  end; //case..of

  Inc(iLoop);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
